/* 29/01/12	version ver 2													*/
/* Changes:																	*/
/*	Hack ad1940 code base												    */
/* 																			*/
/* known Issues: All of them												*/
/* 				  use of arrays is massively code inefficient - fix needed  */



#include <p18f4620.h>
#include "delay.h"
#include "dig_cross.h"
#include "Dig_Cross_EEPROM_Init.h"
#include "Dig_Cross_2012_IC_1.h"
#include "Dig_Cross_2012_IC_1_PARAM.h"
#include "Dig_Cross_2012_IC_1_REG.h"
#include "AD1940_Parm_Map.h"
#include "SigmaStudioFW.h"
#include <math.h>

/* Make this global to this function to save RAM */
int Data_Counter = 0;
double Alpha;
double W0;
double A0, A0_2, IIR_B0_B1_B2_A1_A2[5], IIR2_B0_B1_B2_A1_A2[5];
double TempB0, TempB1;
long int Fixed_Point;
static unsigned char	Read_Data_Array[6];						/* Array used to hold data read during debug process */



/* make this a function, as we need to use a single safeload write function...*/
long int FloatToFixed (double Float_No)
{
long int Fixed_No;

if (Float_No < 0)  				
	{									/* Integer part is float *1^ 23  */
	Fixed_No = -1*(long int)(Float_No * 0x800000);
	Fixed_No = 0x10000000 - Fixed_No;
	}
else
	{									/* Integer part is float *1^ 23  */
	Fixed_No = (long int)(Float_No *  0x800000);
	}
Fixed_No = Fixed_No &0x0FFFFFFF;
return Fixed_No;
}



/****************************************************************************************/
/* this will be done a few times - less RAM to make this a function call				*/
/* This assumes that CSEL is already active 											*/
/****************************************************************************************/
void WriteSPI(long int Data_Word, char bits)
{
unsigned long int Bit_Counter;

    Bit_Counter = ((unsigned long int)1<<(bits-1));		/* start at highest bit */

	for (Data_Counter = bits; Data_Counter > 0; Data_Counter--)
		{ 
		SClkLow;  						/* Set Clock bit Low  */
		if (Bit_Counter & Data_Word)
			PORTC = PORTC | (SDI_Out);      /* Set Data Bit High  */
		else
			PORTC = PORTC & (~SDI_Out);
	
		Bit_Counter = Bit_Counter >> 1;		/* Crank bit to outpu8t down one place */

		SClkHigh;         				/* Set Clock bit High  */
		DelayUs(1);
		}
}


/****************************************************************************************/
/* Input																				*/
/*		devAddress: 		the SPI Address to be read from								*/
/*		Address_to_Read: 	the Address in RAM to be read 								*/
/*		Number_of_Words: 	the number of words											*/
/*		*ptr_to_Data: 		pointer to an array of CHAR in RAM for read data			*/
/*	Output																				*/
/*		Changes data pointed to by *prt_to_Data											*/
/*	Returns																				*/
/*		none																			*/
/****************************************************************************************/
void ReadSPI(char devAddress, int Address_to_Read, int Number_of_Words, unsigned char *ptr_to_Data)
{
int Bit_Counter, Data_Counter;
char Temp_Data; 

CselLow;  											/* Set CSEL Low */

	Write_SPI_Addr((char)devAddress, Read_From_SPI); 	/* Address the DSP */
	Write_16_Bits(Address_to_Read);    					/* Provide the address of data sought */

	for (Data_Counter = 0; Data_Counter < Number_of_Words; Data_Counter++)
		{ 
		Temp_Data = 0;									/* clear temp data holder */
		for (Bit_Counter = 0; Bit_Counter < 8; Bit_Counter++)
			{
			SClkLow;  						/* Set Clock bit Low  */
			DelayUs(10);
			if (SDI_In)	
				Temp_Data++;   		  		/* Set Data Bit High  */
			if (Bit_Counter < 7) 
				Temp_Data = Temp_Data*2;				/* move the bits for the first 7 reads */
			SClkHigh;         				/* Set Clock bit High  */
			DelayUs(10);
			}
		*ptr_to_Data = Temp_Data;
		ptr_to_Data++;
		}

CselHigh;  											/* Set CSEL High */

}



/* Writes data to multiple Device registers    */
/* Note This function is for ROM data only!!!  */
void SIGMA_WRITE_REGISTER_BLOCK(char devAddress, unsigned int address, unsigned int length, rom unsigned char *ptr_to_Data) 
{
unsigned int counter;
rom unsigned char *cPointer;

cPointer = ptr_to_Data;    /* point local variable to the array of data to be written */

CselLow;  											/* Set CSEL Low */
	Write_SPI_Addr((char)devAddress, Write_To_SPI); /* Address the DSP */
	Write_16_Bits(address);          				/* Provide the address of the base of program data */
											  		/* And now do a burst transfer for all program data */
	for(counter = 0; counter < length; counter++)
		{
        Write_8_Bits(*cPointer);
		cPointer++;
		}
CselHigh;         									/* Set CSel bit High  */

DelayUs(More_Than_1_Audio_Frame_uS);

} 


/* Writes data to a Single Device register */
void SIGMA_WRITE_SINGLE_REGISTER(char devAddress, unsigned int address, int length, long int Data) 
{

CselLow;  											/* Set CSEL Low */
	Write_SPI_Addr((char)devAddress, Write_To_SPI); /* Address the DSP */
	Write_16_Bits(address);          				/* Provide the address of the base of program data */
	Write_32_Bits(Data);					  		/* And now write the data */
CselHigh;         									/* Set CSel bit High  */

DelayUs(More_Than_1_Audio_Frame_uS);

} 



/* this will be done a few times - less RAM to make this a function call*/
void SafeLoad_Write_Single_Parameter(unsigned int Address, long int Data)
{
/* write the data */
CselLow;  					 			/* Set CSEL Low */
	Write_SPI_Addr((char)DEVICE_ADDR_IC_1, Write_To_SPI);   		 	/* Address the ADAU1442 */
	Write_16_Bits((unsigned int)ADAU1442_Safeload_Base_Parameter);			/* Provide the address of the indirect safeload parameter register */
	Write_32_Bits(Data);											/* Write data to the AD1940 */
	Write_32_Bits((long int)0);										/* Write data to the AD1940 */
	Write_32_Bits((long int)0);										/* Write data to the AD1940 */
	Write_32_Bits((long int)0);										/* Write data to the AD1940 */
	Write_32_Bits((long int)0);										/* Write data to the AD1940 */
	Write_32_Bits((long int) (Address - 1));						/* Provide the destination address for the data - needs an offset of -1*/
	Write_32_Bits((long int)1);										/* Triggers safeload of one paramater */
CselHigh;								/* Set CSEL High */

/* write the target address*/
//CselLow;  														/* Set CSEL Low */
//	Write_SPI_Addr((char)DEVICE_ADDR_IC_1, Write_To_SPI);   			/* Address the ADAU1442 */
//	Write_16_Bits((int)ADAU1442_Safeload_Target_Address);			/* Provide the address of the indirect safeload address register */
//	Write_32_Bits((long int) (Address - 1));						/* Provide the destination address for the data - needs an offset of -1*/
//CselHigh;  					 		 							/* Set CSEL High */


//CselLow;  					 			/* Set CSEL Low */
//	Write_SPI_Addr((char)DEVICE_ADDR_IC_1, Write_To_SPI);   	/* Address the ADAU1442 */
//	Write_16_Bits((int)ADAU1442_Safeload_Number_Of_Words);			/* Provide the address of the safeload "number_ofwords" register */
//	Write_32_Bits((long int)(1));								/* Triggers safeload of one paramater */
//CselHigh;								/* Set CSEL High */

DelayUs(More_Than_1_Audio_Frame_uS);
}


/* this will be done a few times - less RAM to make this a function call*/
void SafeLoad_Write_IIR_Parms(unsigned int Base_Address, long int W_B2,long int W_B1,long int W_B0,long int W_A2,long int W_A1)
{

/* write the data */
CselLow;  					 			/* Set CSEL Low */
	Write_SPI_Addr(DEVICE_ADDR_IC_1, Write_To_SPI);   		 	/* Address the ADAU1442 */
	Write_16_Bits(ADAU1442_Safeload_Base_Parameter);			/* Provide the address of the indirect safeload parameter register */
	Write_32_Bits(W_B2);										/* Write data to the DSP */
	Write_32_Bits(W_B1);										/* Write data to the DSP */
	Write_32_Bits(W_B0);										/* Write data to the DSP */
	Write_32_Bits(W_A2);										/* Write data to the DSP */
	Write_32_Bits(W_A1);										/* Write data to the DSP */
CselHigh;								/* Set CSEL High */

DelayUs(More_Than_1_Audio_Frame_uS);

/* write the target address*/
CselLow;  														/* Set CSEL Low */
	Write_SPI_Addr(DEVICE_ADDR_IC_1, Write_To_SPI);   			/* Address the ADAU1442 */
	Write_16_Bits(ADAU1442_Safeload_Target_Address);			/* Provide the address of the indirect safeload address register */
	Write_32_Bits((long int) (Base_Address - 1));				/* Provide the destination address for the data, as noted in data sheet need -1 offset */
CselHigh;  					 		 							/* Set CSEL High */

DelayUs(More_Than_1_Audio_Frame_uS);

/* OK, now the filter parameters are ready to go... */
/* need to trigger the safeload						*/
CselLow;  					 			/* Set CSEL Low */
	Write_SPI_Addr(DEVICE_ADDR_IC_1, Write_To_SPI);   		 	/* Address the ADAU1442 */
	Write_16_Bits(ADAU1442_Safeload_Number_Of_Words);			/* Provide the address of the safeload "number_ofwords" register */
	Write_32_Bits((long int)(5));								/* Triggers safeload of five paramaters */
CselHigh;								/* Set CSEL High */

/* the AUAD1442 needs a minimum 1 cycle delay between updates */
DelayUs(More_Than_1_Audio_Frame_uS);
}


/* 
this function is to do the following:
-0- Init registers for configuration of DSP board as built
	- Check source of A/D input (this might in fact be from elsewhere!)
*/
/*	- ADAU1442
		- Hold RESET line active (low) until things settle down
		- WAIT for 256 * MCLK after reset before doing anything! This is about 20uS.
		- Toggle CSEL three times
		- Run through loading registers and RAM and Parameters as per datasheet - if you can make sense of it!!!
*/
void Init_DSP(void)
{
long int Temp_Data;
int counter;
rom unsigned char * Test_Pointer_to_Program;

/* OK, OK, the DSP should already be in reset - but this is to catch errors if this function is called out of context */
SetDSPReset;										/* set Reset_DSP low now - i.e. RESET*/
DelayMs(10);
ClearDSPReset;	
/* Wait about 100mS for things to settle down in the DSP - RESET is low at this point - refer init I/O */
/* Minimum allowable time is 10ms */
DelayMs(100); 
/* toggle CSEL three times minimum - this puts the ADAU1442 into SPI mode*/
for (counter = 0; counter < 4; counter++)
	{
		CselHigh;  DelayUs(10); CselLow; DelayUs(10); 
	}

/* Wait about 100mS for things to settle down in the DSP - RESET is low at this point - refer init I/O */
DelayMs(100); 

/* should be able to turn off clock to Aux ADC,s and ASRC 7:4 */
/* Value is  Ox00EF if you want to do this...  */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_MASTER_CLK_ENABLE_IC_1_ADDR, REG_MASTER_CLK_ENABLE_IC_1_BYTE, R15_MASTER_CLK_ENABLE_IC_1_Default );


//		ReadSPI(DEVICE_ADDR_IC_1, REG_MASTER_CLK_ENABLE_IC_1_ADDR, 2, Read_Data_Array);
//		Print_Data(2, Read_Data_Array, REG_MASTER_CLK_ENABLE_IC_1_ADDR);
//do {} while (1);

/* PHIL - YOU HAVE ADDED THIS TO THIS TO FILE: Dif_Cross_2012_IC_1.h */
//ADI_REG_TYPE R19_STOP_PULSE_SELECT_IC_1_Default[REG_START_PULSE_SELECT_IC_1_BYTE] = {
//0x00, 0x1C
//};
/*clear the start register bit in the DSP */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_START_PULSE_SELECT_IC_1_ADDR, REG_START_PULSE_SELECT_IC_1_BYTE, R19_STOP_PULSE_SELECT_IC_1_Default );

/* PHIL - YOU HAVE ADDED THIS TO THIS TO FILE: Dif_Cross_2012_IC_1.h */
/* Register Default - IC 1.Start_Pulse_Select */
//ADI_REG_TYPE R20_CORE_STOP_BIT_IC_1_Default[REG_CORE_RUN_BIT_IC_1_BYTE] = {
//0x00, 0x00
//};
/* this stops the DSP  */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_CORE_RUN_BIT_IC_1_ADDR, REG_CORE_RUN_BIT_IC_1_BYTE, R20_CORE_STOP_BIT_IC_1_Default );

/* This sets up the serial input */
/* OxA400 = 																*/
/* LRCLK and BCLK pins enabled   						bit 15 = 1			*/
/* Frame Synch = 50/50									bit 14 = 0 			*/
/* Master Clk  Fsclk									bits 13:10 = 1001	*/
/* Neg Bclk, Neg LRCLK, 24 bit, I2S stereo				bits 9-0 = 0		*/ 
/* Input 0 is enabled, others are really don't care							*/
/* note ports 1-3 have LRclk disabled										*/ 
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_SERIALINPUT0_DEFAULT_IC_1_ADDR , R1_SERIAL_INPUT_MODES_IC_1_SIZE, R1_SERIAL_INPUT_MODES_IC_1_Default );
/* This sets up the serial output */
/* OxA400 = 																*/
/* LRCLK and BCLK pins enabled   						bit 15 = 1			*/
/* Frame Synch = 50/50									bit 14 = 0 			*/
/* Master Clk  Fsclk									bits 13:10 = 1001	*/
/* Neg Bclk, Neg LRCLK, 24 bit, I2S stereo				bits 9-0 = 0		*/ 
/* Output 0-3 are enabled, others are really don't care						*/
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_SERIALOUTPUT0_DEFAULT_IC_1_ADDR , R2_SERIAL_OUTPUT_MODES_IC_1_SIZE, R2_SERIAL_OUTPUT_MODES_IC_1_Default );
/* high speed serial slave interface disabled */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_HIGH_SPEED_SLAVE_INTERFACE_MODE_IC_1_ADDR, REG_HIGH_SPEED_SLAVE_INTERFACE_MODE_IC_1_BYTE, R3_HIGH_SPEED_SLAVE_INTERFACE_MODE_IC_1_Default );
/* ASRC 0 input is SPDIF in  */
/* ASRC 1-7 input is No Input  */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_ASRCINPUTSELECTPAIR0_DEFAULT_IC_1_ADDR , R4_ROUTING_MATRIX_ASRC_INPUT_DATA_SELECTOR_IC_1_SIZE, R4_ROUTING_MATRIX_ASRC_INPUT_DATA_SELECTOR_IC_1_Default );
/* ASRC 0-7 input is No Rate  */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_ASRCOUTPUTRATESELECTORPAIR0_DEFAULT_IC_1_ADDR , R5_ROUTING_MATRIX_ASRC_OUTPUT_RATE_SELECTOR_IC_1_SIZE, R5_ROUTING_MATRIX_ASRC_OUTPUT_RATE_SELECTOR_IC_1_Default );
/*	Output 0 = Pair 0 (channel 0,1)		*/
/*	Output 1 = Pair 1 (channel 2,3)		*/
/*	Output 2 = Pair 2 (channel 4,5)		*/
/*	Output 3 = Pair 3 (channel 6,7)		*/
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_SERIALOUTSELECTORPAIR0_DEFAULT_IC_1_ADDR , R6_ROUTING_MATRIX_SERIAL_OUTPUT_DATA_SELECTOR_IC_1_SIZE, R6_ROUTING_MATRIX_SERIAL_OUTPUT_DATA_SELECTOR_IC_1_Default );
/* Diagnostic */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_SPDIF_RECEIVER__READ_AUXILIARY_OUTPUTS_IC_1_ADDR , R7_SPDIF_REGS_IC_1_SIZE, R7_SPDIF_REGS_IC_1_Default );
/* I am not completely convinced by the way the mute and ramp registers are set at this point - mar 2012 */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_ASRC1_MUTE_IN_OUT_IC_1_ADDR, REG_ASRC1_MUTE_IN_OUT_IC_1_BYTE, R8_ASRC1_MUTE_IN_OUT_IC_1_Default );
/* I am not completely convinced by the way the mute and ramp registers are set at this point - mar 2012 */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_ASRC2_MUTE_IN_OUT_IC_1_ADDR, REG_ASRC2_MUTE_IN_OUT_IC_1_BYTE, R9_ASRC2_MUTE_IN_OUT_IC_1_Default );
/* TDM Interface not used*/
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_SERIAL_INPUT_FLEXIBLE_TDM_TO_DSP_CHANNEL_0_IC_1_ADDR , R10_SERIAL_IN_FLEX_MODES_IC_1_SIZE, R10_SERIAL_IN_FLEX_MODES_IC_1_Default );
/* TDM Interface not used*/
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_SERIAL_OUTPUT_FLEXIBLE_TDM_INTERFACE_MODES_0_IC_1_ADDR , R11_SERIAL_OUT_FLEX_MODES_IC_1_SIZE, R11_SERIAL_OUT_FLEX_MODES_IC_1_Default );
/* CRC setup - per Sigma Studio data */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_CRC_IDEAL_1_IC_1_ADDR , R12_DSP_REGS_IC_1_SIZE, R12_DSP_REGS_IC_1_Default );
/* Clocks are output for domains 8 - 3					*/
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_CLOCK_PAD_MULTIPLEXER_IC_1_ADDR , R13_OTHER_DSP_MODES_1_IC_1_SIZE, R13_OTHER_DSP_MODES_1_IC_1_Default );
/* A couple of the pad drive strengths are low vs high - should not be an issue but check if issues seen - mar 2012 */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_BIT_CLOCK_PAD_STRENGTH_IC_1_ADDR , R14_OTHER_DSP_MODES_2_IC_1_SIZE, R14_OTHER_DSP_MODES_2_IC_1_Default );
/* should be able to turn off clock to Aux ADC,s and ASRC 7:4 */
/* Value is  Ox00EF if you want to do this...  */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_MASTER_CLK_ENABLE_IC_1_ADDR, REG_MASTER_CLK_ENABLE_IC_1_BYTE, R15_MASTER_CLK_ENABLE_IC_1_Default );
/* this is really loading the default parameters, and is only necessary to load the Sigma Studio generated address vectors in here. */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, PARAM_ADDR_IC_1, PARAM_SIZE_IC_1, Param_Data_IC_1 );
/* not sure about this address - generated by Sigma Studio */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, R17_IC_1_ADDR_IC_1, R17_IC_1_IC_1_SIZE, R17_IC_1_IC_1_Default );

/* This loads the DSP code into the ADAU1442 */ 
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, PROGRAM_ADDR_IC_1, PROGRAM_SIZE_IC_1, Program_Data_IC_1 );
/* This is in the sigma Studio code - not referred to in the docco. Not aparrently essential..*/
//SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, PROGRAM_ADDR_IC_1, 4, Program_Data_IC_1 );

/* not used in this design */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, NON_MODULO_RAM_ADDR, NON_MODULO_RAM_SIZE, NON_MODULO_RAM_Data );
/* Sets the register start bit to internally generated start pulse */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_DEJITTER_WINDOW_IC_1_ADDR, REG_DEJITTER_WINDOW_IC_1_BYTE, R21_DEJITTER_WINDOW_IC_1_Default );
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_DEJITTER_WINDOW_IC_1_ADDR, REG_DEJITTER_WINDOW_IC_1_BYTE, R22_DEJITTER_WINDOW_IC_1_Default );
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_START_PULSE_SELECT_IC_1_ADDR, REG_START_PULSE_SELECT_IC_1_BYTE, R19_START_PULSE_SELECT_IC_1_Default );
/* this starts the DSP */
SIGMA_WRITE_REGISTER_BLOCK( DEVICE_ADDR_IC_1, REG_CORE_RUN_BIT_IC_1_ADDR, REG_CORE_RUN_BIT_IC_1_BYTE, R20_CORE_RUN_BIT_IC_1_Default );

DelayMs(30);										/* then wait 30ms */ 


/*--------------------------------------------- TESTCODE ------------------------------------------------*/
//ReadSPI(char devAddress, int Address_to_Read, int Number_of_Words, char *ptr_to_Data)
//Print_Data(int Number_of_Words, char *ptr_to_Data)

//for (counter = 0; counter < 2000; counter++)
//	{
//		ReadSPI(DEVICE_ADDR_IC_1, counter, 4, Read_Data_Array);
//		Print_Data(4, Read_Data_Array, counter);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//		DelayMs(255);
//	}
//do {} while (1);



}


/* Initialise ADCs  */
/* not using SPI control of these - so simple hold in reset till the clocks are stable */
/* the CS4398 in particular is pretty fussy about this */
void Init_ADC(void)
{
SetDACReset;
DelayMs(DAC_Reset_Time);		/* Hold ADC and DAC in reset for a defined period of time */
ClearDACReset;
}



/*
Write_Filter_Parm() takes a given Band, Frequency, XO type, Sample Rate and programs the DSP for this
	- For safeload leave bit 9 and 6 as is
	- Program Parameters using SAFELOAD registers
	- Set Bit 4 of Core_Ctrl high - clears itself - initialtes transfer of safeload parameters to parameter RAM

Band:
	- 0X00 bands 0 Upper Frequency
	- 0X01 bands 1 Upper Frequency
	- 0X02 bands 2 Upper Frequency
	- 0X03 bands 3 Upper Frequency
	- 0X10 bands 0 Lower Frequency
	- 0X11 bands 1 Lower Frequency
	- 0X12 bands 2 Lower Frequency
	- 0X13 bands 3 Lower Frequency
Frequency 
	- long int, value in Hz
XO Type
	- 0X00 = No filter	
	- 0X01 = 6dB Buttterworth	
	- 0X02 = 12dB Butterworth	
	- 0X03 = 24dB LR
	- 0X10 = No Filter
	- 0X11 = Peaking (Para EQ)
	- 0X12 = High Shelf
	
Q
	- Q for peaking and High Shelf

Filter_Gain
	- Gain for Peaking and high Shelf filters

Sample_Rate
	- unsigned int, value in samples per second
*/
void Write_Filter_Parm(unsigned char band, unsigned int Frequency, unsigned char XO_Type,  unsigned int Sample_Rate, char Q, char Filter_Gain)
{
int Temp_Address1, Temp_Address2;
int counter;
double SinW0, CosW0;
double Gain_A, Alpha_on_A, Alpha_by_A, A_Plus_1, A_Minus_1, SQRT_2_A;

	/* Filter_Gain is the peaking gain or high shelf gain parameter */
	/* this value of "A" - gain for these filters ia 10^(boos/40)   */
	/* go figure...  what a bastard of a value. 					*/
	/* for 4th order butterworth, need to cascade two filters		*/
	/*					1st Q = 0.54								*/
	/*					2nd Q = 1.31								*/
if (Filter_Gain <= 0)
	{
	Gain_A = ((float)(Atten_Shift_Para[-Filter_Gain]))/255.0;
	}
else
	{
	Gain_A = 255.0/((float)Atten_Shift_Para[Filter_Gain]);
	}

/* W0 = 2*pi*f0/Fs */
W0 = 2 * Pi * (double)Frequency / (double)Sample_Rate;

/* Calculate SinW0 and CosW0  - this is cheaper on memory than the library routine in the compiler...*/

	A0 = W0*W0;   /* borrow A0 */
	SinW0 = -2.39e-8;
	SinW0 = SinW0 * A0;
	SinW0 = SinW0 + 2.7526e-6;
    SinW0 = SinW0 * A0;
    SinW0 = SinW0 - 1.98409e-4;
    SinW0 = SinW0 * A0;
    SinW0 = SinW0 + 8.3333315e-3;
    SinW0 = SinW0 * A0;
    SinW0 = SinW0 - 1.666666664e-1;
    SinW0 = SinW0 * A0;
    SinW0 = SinW0 + 1.0;
    SinW0 = SinW0 * W0;

	CosW0 = (float)-2.605e-7;
	CosW0 *= A0;
	CosW0 += (float) 2.47609e-5;
	CosW0 *= A0;
	CosW0 -= (float) 1.3888397e-3;
	CosW0 *= A0;
	CosW0 += (float) 4.16666418e-2;
	CosW0 *= A0;
	CosW0 -= (float) 4.999999963e-1;
	CosW0 *= A0;
	CosW0 += (float) 1.0;

Alpha = 0;
/* Case of NO filter "none" */
if ((XO_Type == (unsigned char)0) || (XO_Type == (unsigned char)0X10))
	{
	/* If want no filter set B0 = 1, rest = 0 */
	A0 = 0;
	A1 = 0;
	A2 = 0;
	B0 = 1;
	B1 = 0;
	B2 = 0;

	/* If want no filter set B0 = 1, rest = 0 */
	A0_2 = 0;
	A1_2 = 0;
	A2_2 = 0;
	B0_2 = 1;
	B1_2 = 0;
	B2_2 = 0;

	}
/* Case of 6dB filter */
else if (XO_Type == (unsigned char)1)
	{
	/* Low Pass
		a1 =  2.7^-?0
		b0 =  gainLinear * (1.0 - a1)
		b1 =  0   */
	/* High Pass
		a1 =  2.7^-?0
		b0 =  gainLinear * a1
		b1 = -a1 * gainLinear  */	
	A0 = 1;
	/* Algorithm to calculate e power w0) .... 	A1 = pow(2.7, -W0); */
	counter = 1;
	B0 = 1;							/* borrow B0 for a while - use this to calculate X^n - real value stuck in later*/
	B1 = 1;							/* borrow B1 for a while - use this to calculate n! - real value stuck in later*/
	A1 = 1;  						/* first term */
	do								/* This is essentially a taylor series expansion of e ^ -W0 */
		{
		B0 = -B0 * W0;
		B1 = B1 * counter;
		A1 = A1 + B0/B1;
		counter++;
		}
	while (counter < 10);
	A2 = 0;							/* B0 and B1 are always initialised in the following if -- else */
	B2 = 0;	
	if (band < (unsigned char)8)
		{
		B0 = 1 - A1;
		B1 = 0;
		}
	else
		{
		B0 = A1;
		B1 = -A1;
		}

		/* for 6dB filter, the second filter set need to be set to flat */
		/* If want no filter set B0 = 1, rest = 0 */
		A0_2 = 0;
		A1_2 = 0;
		A2_2 = 0;
		B0_2 = 1;
		B1_2 = 0;
		B2_2 = 0;

	}
else if ((XO_Type == (unsigned char)2) || (XO_Type == (unsigned char)3))
	{
	/*Butterworth LP and HP  alpha = sin(?0) / 2.0 * 1/sqrt(2) */	
	Alpha =  SinW0/sqrt2;
	/* these are the same for both LP and HP */
	A0 = 1 + Alpha;
	A1 = (2*CosW0)/A0;
	A2 = (Alpha-1)/A0;
	if (band < (unsigned char)8)   /* The crossover is a low pass if band < 8 */
		{
		/*	a0 =   1 + alpha
			a1 =  -2*cos(?0)
			a2 =   1 - alpha
			b0 =  (1 - cos(w0)) * gainLinear / 2
			b1 =   1 - cos(w0)  * gainLinear
			b2 =  (1 - cos(?0)) * gainLinear / 2  */
		B1 = (1 - CosW0)/(A0);
		B0 = B1/2;
		B2 = B0;
		}
	else  /* The crossover is a high pass if band > 8 */
		{
		B1 = -(1 + CosW0)/(A0);
		B0 = -B1/2;
		B2 = B0;
		}

		/* for 12 and 24dB filters, the second filter set need to be set to flat */
		/* If want no filter set B0 = 1, rest = 0 */
		A0_2 = 0;
		A1_2 = 0;
		A2_2 = 0;
		B0_2 = 1;
		B1_2 = 0;
		B2_2 = 0;
	}

/************************************************************************************************/
/* This is only used for 8th order LR filters													*/
/************************************************************************************************/
else if (XO_Type == (unsigned char)4)
	{
	/* 4th order Butterworth LP and HP  alpha = sin(w0) / 2.0 * Q */	
	/* do the first set of filters, Q = 0.54 */
	Alpha =  SinW0/(2.0*0.54);
	/* these are the same for both LP and HP */
	A0 = 1 + Alpha;
	A1 = (2*CosW0)/A0;
	A2 = (Alpha-1)/A0;
	if (band < (unsigned char)8)   /* The crossover is a low pass if band < 8 */
		{
		/*	a0 =   1 + alpha
			a1 =  -2*cos(?0)
			a2 =   1 - alpha
			b0 =  (1 - cos(w0)) * gainLinear / 2
			b1 =   1 - cos(w0)  * gainLinear
			b2 =  (1 - cos(?0)) * gainLinear / 2  */
		B1 = (1 - CosW0)/(A0);
		B0 = B1/2;
		B2 = B0;
		}
	else  /* The crossover is a high pass if band > 8 */
		{
		B1 = -(1 + CosW0)/(A0);
		B0 = -B1/2;
		B2 = B0;
		}

	/* 4th order Butterworth LP and HP  alpha = sin(w0) / 2.0 * Q */	
	/* do the second set of filters, Q = 1.31 */
	Alpha =  SinW0/(2.0*1.31);
	/* these are the same for both LP and HP */
	A0_2 = 1 + Alpha;
	A1_2 = (2*CosW0)/A0_2;
	A2_2 = (Alpha-1)/A0_2;
	if (band < (unsigned char)8)   /* The crossover is a low pass if band < 8 */
		{
		B1_2 = (1 - CosW0)/(A0_2);
		B0_2 = B1_2/2;
		B2_2 = B0_2;
		}
	else  /* The crossover is a high pass if band > 8 */
		{
		B1_2 = -(1 + CosW0)/(A0_2);
		B0_2 = -B1_2/2;
		B2_2 = B0_2;
		}
	}


else if (XO_Type == (unsigned char)0x11)
	{
	/*Peaking - Parametric EQ Filter*/	
	/* Remember that Q is in units of Q*10 */
	Alpha =  SinW0/(2.0 * (double)Q / 10.0);

	Alpha_on_A = Alpha / Gain_A;  
	Alpha_by_A = Alpha * Gain_A;

	/* these are the same for both LP and HP */
	A0 = 1 + Alpha_on_A;
	A1 = (2*CosW0)/A0;
	A2 = (Alpha_on_A - 1)/A0;

	B0 = (1 + Alpha_by_A)/A0;
	B1 = -(2* CosW0)/(A0);
	B2 = (1 - Alpha_by_A)/A0;;
	}

else if (XO_Type == (unsigned char)0x12)
	{
	/* High Shelf - CD Horn Correction*/	
	/* Q of 0.7 and S = 1 degenerates to the following */
	Alpha =  SinW0/sqrt2;

	/* These values for A+1 and A-1 are for A = 15dB */
	A_Plus_1 = 3.371373706;
	Gain_A = 2.371373706;
	A_Minus_1 = 1.371373706;
	SQRT_2_A = 2.177785;
	
	/* Coeficients for the High Shelf Filter */
	A0 = A_Plus_1 - A_Minus_1*CosW0 + SQRT_2_A*SinW0;
	A1 = -2*(A_Minus_1 - A_Plus_1*CosW0)/A0;
	A2 = -(A_Plus_1 - A_Minus_1*CosW0 - SQRT_2_A*SinW0)/A0;

	B0 = Gain_A * (A_Plus_1 + A_Minus_1*CosW0 + SQRT_2_A*SinW0)/A0;
	B1 = -2*Gain_A *(A_Minus_1 + A_Plus_1*CosW0)/A0;
	B2 = Gain_A * (A_Plus_1 + A_Minus_1*CosW0 - SQRT_2_A*SinW0)/A0;
	}

else;

if (XO_Type < (unsigned char)0x10)   /* Then we are programming the XO filters */
	{
										/* char Filter_Map[4][2][2][5] */ 
										/* [Band] [Fh, Fl] [Filter1, Filter2, Filter3, Filter4] [b0, b1, b2, a1, a2] */
										/* void(int Address, double Data, char offset)  */
										/* Safeload Parameters */
										/* Write the 5 words to the safeload registers*/
	Temp_Address1 = band & 0x03;		/* do this to save instructions */
	Temp_Address2 = (band & 0x10) >> 4;	/* do this to save instructions */

	/* this function loads and initiates safeload write of IIR filter parameters to DSP 			*/
	/* Need to provide address of first parameter - B2 for this DSP, then data B2, B1, B0, A2, A1	*/
	SafeLoad_Write_IIR_Parms((unsigned int) (Filter_Map[Temp_Address1][Temp_Address2][0][2]), FloatToFixed(IIR_B0_B1_B2_A1_A2[2]), FloatToFixed(IIR_B0_B1_B2_A1_A2[1]), FloatToFixed(IIR_B0_B1_B2_A1_A2[0]), FloatToFixed(IIR_B0_B1_B2_A1_A2[4]),FloatToFixed(IIR_B0_B1_B2_A1_A2[3]));

	if (XO_Type <= (unsigned char)2)					/* Write second lot of parameters.... */
		{
			B0 = 1;
			B1 = 0;
			B2 = 0;
			A0 = 0;
			A1 = 0;
			A2 = 0;
		}

	/* this function loads and initiates safeload write of IIR filter parameters to DSP */
	SafeLoad_Write_IIR_Parms((unsigned int) (Filter_Map[Temp_Address1][Temp_Address2][1][2]), FloatToFixed(IIR_B0_B1_B2_A1_A2[2]), FloatToFixed(IIR_B0_B1_B2_A1_A2[1]), FloatToFixed(IIR_B0_B1_B2_A1_A2[0]), FloatToFixed(IIR_B0_B1_B2_A1_A2[4]),FloatToFixed(IIR_B0_B1_B2_A1_A2[3]));

	/* this function loads and initiates safeload write of IIR filter parameters to DSP */
	SafeLoad_Write_IIR_Parms((unsigned int) (Filter_Map[Temp_Address1][Temp_Address2][2][2]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[2]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[1]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[0]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[4]),FloatToFixed(IIR2_B0_B1_B2_A1_A2[3]));

	/* this function loads and initiates safeload write of IIR filter parameters to DSP */
	SafeLoad_Write_IIR_Parms((unsigned int) (Filter_Map[Temp_Address1][Temp_Address2][3][2]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[2]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[1]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[0]), FloatToFixed(IIR2_B0_B1_B2_A1_A2[4]),FloatToFixed(IIR2_B0_B1_B2_A1_A2[3]));

	}
else     /* Else we are programming the parametric filters */
	{
	/* this function loads and initiates safeload write of IIR filter parameters to DSP */
	SafeLoad_Write_IIR_Parms((unsigned int) Para_Filter_Map[band][2], FloatToFixed(IIR_B0_B1_B2_A1_A2[2]), FloatToFixed(IIR_B0_B1_B2_A1_A2[1]), FloatToFixed(IIR_B0_B1_B2_A1_A2[0]), FloatToFixed(IIR_B0_B1_B2_A1_A2[4]),FloatToFixed(IIR_B0_B1_B2_A1_A2[3]));
	}
}


/*
Write_Input() takes a given input select - eitherSPDIF or Input zero to the ADAU1442
*/
void Write_Input(char Input_Selected)
{

if (Input_Selected == (char)Input_ADC)  /* the input is the ADC */
	{
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, MOD_INPUT_SEL_ALG0_STAGE0_STEREOSWITCHNOSLEW_ADDR, NumBytes_Word_Parms, MOD_INPUT_SEL_ALG0_STAGE1_STEREOSWITCHNOSLEW_FIXPT );
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, MOD_INPUT_SEL_ALG0_STAGE1_STEREOSWITCHNOSLEW_ADDR, NumBytes_Word_Parms, MOD_INPUT_SEL_ALG0_STAGE0_STEREOSWITCHNOSLEW_FIXPT );	
	}
else    /* the input is the SPDIF input */
	{
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, MOD_INPUT_SEL_ALG0_STAGE0_STEREOSWITCHNOSLEW_ADDR, NumBytes_Word_Parms, MOD_INPUT_SEL_ALG0_STAGE0_STEREOSWITCHNOSLEW_FIXPT );
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, MOD_INPUT_SEL_ALG0_STAGE1_STEREOSWITCHNOSLEW_ADDR, NumBytes_Word_Parms, MOD_INPUT_SEL_ALG0_STAGE1_STEREOSWITCHNOSLEW_FIXPT );	
	}
}



/*
Write_Invert() Inverts selected band
*/
void Write_Invert(char band, char Invert)
{

if (Invert == Invert_True)  /* User wants band inverted */
	{
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, Invert_Map[band*2], NumBytes_Word_Parms, Inverted_Gain_Val );
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, Invert_Map[band*2 + 1], NumBytes_Word_Parms, Inverted_Gain_Val );	
	}
else    /* Don't invert band... */
	{
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, Invert_Map[band*2], NumBytes_Word_Parms, Non_Inverted_Gain_Val );
		SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, Invert_Map[band*2 + 1], NumBytes_Word_Parms, Non_Inverted_Gain_Val );	
	}
}



/*
Write_DC_Block() Uses Sigma Studio  Values to Set DC Block
*/
void Write_DCBlock(void)
{
	SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, MOD_DCB1_DCBALGDOUPREC1POLE_ADDR, NumBytes_Word_Parms, MOD_DCB1_DCBALGDOUPREC1POLE_FIXPT );
	SIGMA_WRITE_SINGLE_REGISTER( DEVICE_ADDR_IC_1, MOD_DCB2_DCBALGDOUPREC2POLE_ADDR, NumBytes_Word_Parms, MOD_DCB2_DCBALGDOUPREC2POLE_FIXPT );
}



/*
Write_Atten() takes a given bands required attenuation and programs the DSP for this
	- For safeload leave bit 9 and 6 as is
	- Program Parameters using SAFELOAD registers
	- Set Bit 4 of Core_Ctrl high then low (initialte safeload transfer to parameter RAM)
Band:
	- 0X00 bands 0 
	- 0X01 bands 1
	- 0X02 bands 2
	- 0X03 bands 3
Attenuation:
	- unsigned char 
How the hell does this work?
	- The DSP simply implements the attenuation by multiplying the signal with a constant
	- The value (unsurprisingly) is a 5.24 fixed point value
	- Attenuation is value < 1, or in fixed point 0x080000
	- This function simply implements this by - well an approximation....
		- The attenuation in dB is divided by 6, which is a right shift of the value by one place
		- So the "6db steps" of attenuation are first set up by 0x800000 >> Aten/6dB
		- Then the in between bits (0-5dB) need to be added back in....
*/
void Write_Atten(char band, unsigned char atten)
{
long int Atten_Calc;
long int Interim_Atten_Calc;

/* apply a reasonable limit to the attenuation */
if (atten > (unsigned char)180)
	atten = 180;

/* Coarse attenuation in 6dB - remember that the attenuation is in 0.5 dB steps */
Atten_Calc = 0x800000 >> (atten/12); 
/* Interim Atten Calc is a constant that will be SUBTRACTED from the coarse 6dB steps */
/* The data from the constan array is an 8 bit number representing the amount less than 255 that the offset in dBs represents */
/* again remember this is in 0.5dB steps, so need to use %12 */
Interim_Atten_Calc = ((long int)Atten_Shift[atten % 12]);
/* This needs to be shifted right to the appropriate bitplace for the 6dB steps */
/* The initial data is already 8 bits wide */
Interim_Atten_Calc = Interim_Atten_Calc << (15 - atten/12);
Atten_Calc = Atten_Calc - Interim_Atten_Calc;
									/* safeload write parameters for first channel */
SafeLoad_Write_Single_Parameter(Atten_Map[2*band], Atten_Calc);
									/* safeload write parameters for second channel */
SafeLoad_Write_Single_Parameter(Atten_Map[(2*band) + 1], Atten_Calc);

}




/*
Write_PGA2320() takes a given bands required attenuation and programs the volume control IC for this

Band:
	This code assumes there are four DACS - if there are not it does not make a sausage of difference as ALL dacs get the same atten
Attenuation:
	- unsigned char 
How the hell does this work?
	- The PGA2320 gain is defined by left and right input data, of 8 bits per channel
	- Gain = 31.5 - (0.5*(255-n)) dB

*/
void Write_PGA2320(unsigned char atten)
{
char loop;

CselLowPGA2320;
for(loop = 0; loop < 8; loop++)
	{
		WriteSPI((long int)atten, 8);
	}
CselHighPGA2320;
}



/*
Write_Delay() takes a bands required delay and programs the DSP for this
	- set bit 9 of Core_Ctrl register set HIGH
	- Program Attenuation Parameter using SAFELOAD
	- Clear Bit 9

Band:
	- 0X00 bands 0 
	- 0X01 bands 1
	- 0X02 bands 2
	- 0X03 bands 3
Delay:
	- long int 
*/
void Write_Delay(unsigned char band, int Delay_mm, unsigned int Sample_Rate)
{
float Delay_Clocks;

Delay_Clocks = (float)Sample_Rate * (float)Delay_mm;
Delay_Clocks = Delay_Clocks / SpeedOfSound;

/* Dont let this be zero though */
if (Delay_Clocks < 1)
	Delay_Clocks = 1;

/* safeload write parameters to AD1940 */
SafeLoad_Write_Single_Parameter(Delay_Map[band],(int)Delay_Clocks);
}


