/* Digital Crossover Controller for ADAU1442. 								*/
/* 29/01/12	version ver 3													*/
/* Changes:																	*/
/* Hack ad1940 code base												    */
/* known Issues: All of them												*/
/* Code still written for AD1940 											*/
/* Compiles and runs on 18F4620												*/
/* 																			*/
/* 15/02/12	version ver 4													*/
/* Changes:																	*/
/* 		EEPROM INIT OK														*/
/* 		Save Data OK														*/
/* 		Menu Structures OK													*/
/* 		Data Structures OK													*/
/* 		Init data ok														*/
/* 		BW24 Parameters and Calcs											*/
/* 																			*/
/* ************************************************************************ */
/* 19/2/12	Ver 5															*/
/* Changes:																	*/
/* 	Added delay after starting ISR to allow RotStatus to stabilise		 	*/
/* 	Stopped erroneous decrement / increment of volume if exit menu			*/
/*	Added Load and Save Invert bit		(needs code for DSP control)		*/
/* 	Add SPDIF / Digitiser input Select	(needs code for DSP control)		*/
/* 																			*/
/* ************************************************************************ */
/* 20/2/12	Ver 6															*/
/* Known Needs: 															*/
/* 		Code to initialise ADAU1442	- done									*/
/* 		Code to load ADAU1442 program - done								*/
/* 		Parameter map for ADAU1442	- done 1st cut							*/
/* 		Invert bit function			- coded, not tested						*/
/* 		Input select function		- coded, not tested						*/
/* ************************************************************************ */
/* 26/2/12	Ver 7															*/
/* Notes: 																	*/
/*		The analog Devices auto generated code needs a couple of tweaks 	*/
/*		prior to compiling into this code:									*/
/*			ADI_REG_TYPE needs to be ROM in SigmaStudioFW.h					*/
/*			Take out definition of macro for SIGMA_WRITE_REGISTER_BLOCK 	*/
/*			in SigmaStudioFW.h												*/
/*																			*/
/* Known Needs: 															*/
/* 		Code to initialise ADAU1442		(coded and checked)					*/
/* 		Code to load ADAU1442 program	(coded and checked)					*/
/* 		Parameter map for ADAU1442 test										*/
/* 		Invert bit function test											*/
/* 		Input select function (coded)										*/
/* 		Safeload Fucntion - test											*/
/* 																			*/
/* The DSP loads and runs using the Analog devices data directly loaded		*/
/* 		Parameter maps loaded (need to be checked)							*/
/*		Voume control code needs to be fixed then checked					*/
/*		Safeload code needs to be done - only just started this				*/
/* 		Output invert code needs to be implemented and checked				*/
/* ************************************************************************ */
/* 26/3/12	Ver 7															*/
/* Notes: 																	*/
/*		Init ADAU1442 - mostly OK											*/
/*		ADAU1442 Parameter map - OK											*/
/*		Invert bit - OK														*/
/*		Innput Select Function - Selects A/D OK, not so clear on SPDIF		*/
/*		Safeload Function - Not really OK, DSP stops on writing data 		*/	
/*					OUTPUT ONLY RESTARTS AFTER WRITE TO GENERAL REGISTER!	*/
/*		Parameter maps - OK													*/
/*		Volume code - OK (though safeload stops DSP somehow)				*/
/*		Parametric EQ's seem OK												*/
/* 		There is some issue with filters - unexpected rolloff at 6KHz		*/
/* ************************************************************************ */
/*	31/3/2012 Ver 10														*/
/*	Notes:																	*/
/*		Init ADAU1442 - OK													*/
/*		Innput Select Function - Selects A/D abd SPDIF OK					*/
/*		Safeload Function - Seems OK	 									*/	
/* 		Unexpected rolloff at 6KHz - isolated ro wiring SPI Clock to DAC...	*/
/*			Need to isolate this wire on current rev of DACS...  OK			*/
/*		SPDIF input working OK												*/
/* ************************************************************************ */
/*																			*/
/*	1/4/2012																*/
/*		Release 1 ver 1-00													*/
/*		Volume works in 0.5dB steps											*/
/*		Band Attenuation works in 0.5dB steps								*/
/*		No known bugs														*/
/*																			*/
/* ************************************************************************ */
/*																			*/
/*	Release 2 Ver 2-00																		*/
/*	24/6/2012																*/
/*		Update software to address PGA2320 on DAC							*/
/*		Needs mods to:					 									*/
/*			- Set ADAU1442 output to someting high (say -3 or 6dB)			*/
/*			- Implement volume control in PGA2320							*/
/*				Use B05 as PGA2320 Select Output							*/
/*																			*/
/* ************************************************************************ */
#include <p18F4620.h>
#include <timers.h>
#include "delay.h"
#include "dig_cross.h"
#include "AD1940.h"

/* Do something gross - and define the data structure such 					*/
/* that it is globally accessible to this module							*/
/*  my excuse it that this allows me to avoid mucking 						*/
/* around with pointers etc  												*/

/* Delete me later please */
void timer_isr (void);
static unsigned char Old_Rot_Status = 0;  			/* Records state of rotary control */
static unsigned char Fast_Loop_Counter = Fast_Counter_Init; 		/* Used to set Fast_Cnt */
static unsigned char Slow_Loop_Counter = Slow_Counter_Init; 		/* Used to see if user turning control continuously */
static unsigned char Keypress_Read = 0;				/* this is the data on key / rotation changes */
static unsigned char Key_Just_Pressed = 0;  		/* use this to keep track of which keys have been pressed */
static unsigned char Fast_Change = 1;				/* use this to indicate fast change of values */

data_val_struct Data_Values[4];
data_val_struct *Data_Values_Pointer, *Temp_Data_Values_Pointer;
Para_EQ_struct EQ_Values[15];						/* This structure holds all the parametrc filter values 3 common filter sets, plus 2 per band */
Para_EQ_struct *EQ_Values_Pointer, *Temp_EQ_Values_Pointer;
char loop, test_temp;

/* You might want to move these into the display LCD function - though it is really not a gut buster for these two */
char line1[] =   Line1_Init;
char line2[] =   Line2_Init;
char *line1_ptr, *line2_ptr;						/* use these to point into the arrays */

/* These are ROM constants used in bnuilding the display 					*/
rom char Blank_Char = Blank_Char_Init;    								/* ROM */
rom char Sel_Str[] = Line2_Select_Init;    								/* ROM */
int display_temp;
rom char EQ_Str[] =  Line1_EQ_Init;    									/* ROM */
rom char XO_Str[] =  Line1_XO_Init;    									/* ROM */
rom char Save_Str[] =  Line1_Save_Init;									/* ROM */
rom char Load_Str[] =  Line1_Load_Init;									/* ROM */
rom char Choose_Str[] = Line1_Choose_Str_Init; 							/* ROM */
rom char Vol_Str[] = Line1_Vol_Init;									/* ROM */
rom char Vol2_Str[] = Line2_Vol_Init;									/* ROM */
rom char Input_Str[] = Line1_Input_Init;								/* ROM */
rom char Slopes[5][4] = Slopes_Def_Init;								/* ROM */

rom char XO_Choice_Display[8][17] =
{
"XO -> Band:     ",
"Band X, Low Frq:",
"Band X, Hi Frq: ",
"Band X, Low XO: ",
"Band X, High XO:",
"Band X, Delay:  ",
"Band X, Atten:  ",
"Band X, Invert: "
};																	/* ROM */

rom char XO_Units_Display[6][6] =
{
"Hz   ",
"Hz   ",
"Slope",
"Slope",
"mm   ",
"dB   "
};																	/* ROM */

rom char Choice_Display[6][17] =
{
"Volume          ",
"Crossover       ",
"Save Parameters?",
"Load Parameters?",
"EQ Setup        ",
"Input Select    "
};																	/* ROM */


rom char EQ_Choice_Display[2][6] =
{
"Band ",
"Comm "
};																	/* ROM */

rom char EQ_Select_Display[4][12] =
{
"X, EQ Freq:",
"X, EQ Q:   ",
"X, EQ Gain:",
"X, EQ Type:"
};																	/* ROM */

rom char EQ_Units_Display[4][6] =
{
"Hz   ",
"     ",
"dB   ",
"     "
};																	/* ROM */

rom char EQ_Type_Display[4][11] =
{
"None      ",
"Parametric",
"CD Horn   "
};	

rom char Yes_No[2][4] = 
{
"Yes",
"No "
};

rom char Input_Choices[2][17] = 
{
"SPDIF           ",
"A/D Converter   "
};


/****************************************************************************/
/* Initialise the PIC														*/
/****************************************************************************/

void Init_IO(void)
{

/* ADCON setup */
	ADCON0 = 0;				// A/D Port reconfiguration
	ADCON1 = 0x0f;			// A/D Port reconfiguration
	ADCON2 = 0;				// A/D Port reconfiguration

/* set up interrupts */
	PIE1 = 0;				// Disable interrupt 1
	PIE2 = 0;				// Disable interrupt 2
	INTCONbits.GIE = 0;		// Global interrupt disable


/* Refer to Dif Cross.h for definitions
* Port A = bit 0-4 Inputs
* Port B Output - character selects 
* Port C bit 4 is an Input
* Port D Output - mul	tiplexed leds
* Port E outputs `
*/
/* initialise IO */
	TRISA=0X1E;
	TRISB=0X00;
	TRISC=0X10;
	TRISD=0X00;
	TRISE=0x00;


/* disable CSEL */
PORTC = PORTC | (CSel);                    /* Set Clock bit High  */
/* Hold DSP in RESET */
PORTC = PORTC & (255 - Reset_DSP);         /* Hold DSP Reset Low  */
SetDACReset;							   /* Hold DAC in RESET */
PORTB = 0;   
CselHighPGA2320;
}



/****************************************************************************/
/* Do this in one place  -  save memory and hassles with consistency!    	*/
/****************************************************************************/
void Load_From_Memory(char Mem_Bank)
{
int BandTemp;

/* Read in the parameters for crossover points from default bank of EEPROM */
for (loop=0; loop < 4; loop++)
	{
											/* Use pointers to the data structures */
											/* as it is more code efficient */
	Data_Values_Pointer = &(Data_Values[loop]);
												/* Read Data for each band from EEPROM */
	BandTemp = (loop * Band_Data_Array_Size) + (ParmSet_Array_Size * Mem_Bank);	
											/* Lower crossover Freq */
	Data_Values_Pointer->Fl = EEPROM_READ(BandTemp + Fl_Offset);						
	Data_Values_Pointer->Fl += EEPROM_READ(BandTemp + Fl_Offset + 1) * 256;
											/* Upper crossover Freq */
	Data_Values_Pointer->Fu = EEPROM_READ(BandTemp + Fu_Offset);						
	Data_Values_Pointer->Fu += EEPROM_READ(BandTemp + Fu_Offset + 1) * 256;
											/* Lower crossover Slope */
	Data_Values_Pointer->Sl = EEPROM_READ(BandTemp + Sl_Offset);
											/* Upper crossover Slope */
	Data_Values_Pointer->Su = EEPROM_READ(BandTemp + Su_Offset);
											/* Attenuation to be applied to band */
	Data_Values_Pointer->Atten = EEPROM_READ(BandTemp + Atten_Offset);
											/* Delay to be applied to band */
	Data_Values_Pointer->Delay_mm = EEPROM_READ(BandTemp + Delay_mm_Offset);
	Data_Values_Pointer->Delay_mm += EEPROM_READ(BandTemp + Delay_mm_Offset + 1) *256;
	
	EQ_Values_Pointer = &(EQ_Values[Band_1_Para_data + loop*3]);   /* use this to point at the EQ set */

										/* Parametric EQ # 1 Centre Freq */
	EQ_Values_Pointer->CF = EEPROM_READ(BandTemp + Para_CF_Offset);			
	EQ_Values_Pointer->CF += EEPROM_READ(BandTemp + Para_CF_Offset + 1) *256;
											/* Parametric EQ Q Value */
	EQ_Values_Pointer->Q = EEPROM_READ(BandTemp + Para_Q_Offset);			
											/* Parametric EQ Gain */
	EQ_Values_Pointer->Gain = EEPROM_READ(BandTemp + Para_Gain_Offset);			
											/* Parametric EQ Type */
	EQ_Values_Pointer->Type = EEPROM_READ(BandTemp + Para_Type_Offset);			

	EQ_Values_Pointer = &(EQ_Values[Band_1_Para_data + loop*3 + 1]);   
											/* use this to point at the EQ set, second para follows the first... */
											/* Parametric EQ # 2 Centre Freq */
	EQ_Values_Pointer->CF = EEPROM_READ(BandTemp + Para_CF_1_Offset);			
	EQ_Values_Pointer->CF += EEPROM_READ(BandTemp + Para_CF_1_Offset + 1) *256;
											/* Parametric EQ Q Value */
	EQ_Values_Pointer->Q = EEPROM_READ(BandTemp + Para_Q_1_Offset);			
											/* Parametric EQ Gain */
	EQ_Values_Pointer->Gain = EEPROM_READ(BandTemp + Para_Gain_1_Offset);			
											/* Parametric EQ Type */
	EQ_Values_Pointer->Type = EEPROM_READ(BandTemp + Para_Type_1_Offset);			


	EQ_Values_Pointer = &(EQ_Values[Band_1_Para_data + loop*3 + 2]);   
											/* use this to point at the EQ set, second para follows the first... */
											/* Parametric EQ # 3 Centre Freq */
	EQ_Values_Pointer->CF = EEPROM_READ(BandTemp + Para_CF_2_Offset);			
	EQ_Values_Pointer->CF += EEPROM_READ(BandTemp + Para_CF_2_Offset + 1) *256;
											/* Parametric EQ Q Value */
	EQ_Values_Pointer->Q = EEPROM_READ(BandTemp + Para_Q_2_Offset);			
											/* Parametric EQ Gain */
	EQ_Values_Pointer->Gain = EEPROM_READ(BandTemp + Para_Gain_2_Offset);			
											/* Parametric EQ Type */
	EQ_Values_Pointer->Type = EEPROM_READ(BandTemp + Para_Type_2_Offset);			
	Data_Values_Pointer->Invert = EEPROM_READ(BandTemp + Invert_Offset);
											/* Is the band to be inverted or not? */

	};


/* Read Data for each Para Set from EEPROM */
BandTemp = (Para_Gen_Base + Para_Parmset_Size * Mem_Bank);	

									/* Use pointers to the data structures */
									/* as it is more code efficient */
EQ_Values_Pointer = &(EQ_Values[Gen1_Para_data]);

/* Gen1 Parametric EQ Centre Freq */
EQ_Values_Pointer->CF = EEPROM_READ(BandTemp + Para_Gen1_CF_Offset);			
EQ_Values_Pointer->CF += EEPROM_READ(BandTemp + Para_Gen1_CF_Offset + 1) *256;
/* Gen1 Parametric EQ Q Value */
EQ_Values_Pointer->Q = EEPROM_READ(BandTemp + Para_Gen1_Q_Offset);			
/* Gen1 Parametric EQ Gain */
EQ_Values_Pointer->Gain = EEPROM_READ(BandTemp + Para_Gen1_Gain_Offset);			
/* Gen1 Parametric EQ Type */
EQ_Values_Pointer->Type = EEPROM_READ(BandTemp + Para_Gen1_Type_Offset);			

									/* Use pointers to the data structures */
									/* as it is more code efficient */
EQ_Values_Pointer = &(EQ_Values[Gen2_Para_data]);

/* Gen2 Parametric EQ Centre Freq */
EQ_Values_Pointer->CF = EEPROM_READ(BandTemp + Para_Gen2_CF_Offset);			
EQ_Values_Pointer->CF += EEPROM_READ(BandTemp + Para_Gen2_CF_Offset + 1) *256;
/* Gen1 Parametric EQ Q Value */
EQ_Values_Pointer->Q = EEPROM_READ(BandTemp + Para_Gen2_Q_Offset);			
/* Gen1 Parametric EQ Gain */
EQ_Values_Pointer->Gain = EEPROM_READ(BandTemp + Para_Gen2_Gain_Offset);			
/* Gen1 Parametric EQ Type */
EQ_Values_Pointer->Type = EEPROM_READ(BandTemp + Para_Gen2_Type_Offset);			
	

									/* Use pointers to the data structures */
									/* as it is more code efficient */
EQ_Values_Pointer = &(EQ_Values[Gen3_Para_data]);

/* Gen3 Parametric EQ Centre Freq */
EQ_Values_Pointer->CF = EEPROM_READ(BandTemp + Para_Gen3_CF_Offset);			
EQ_Values_Pointer->CF += EEPROM_READ(BandTemp + Para_Gen3_CF_Offset + 1) *256;
/* Gen1 Parametric EQ Q Value */
EQ_Values_Pointer->Q = EEPROM_READ(BandTemp + Para_Gen3_Q_Offset);			
/* Gen1 Parametric EQ Gain */
EQ_Values_Pointer->Gain = EEPROM_READ(BandTemp + Para_Gen3_Gain_Offset);			
/* Gen1 Parametric EQ Type */
EQ_Values_Pointer->Type = EEPROM_READ(BandTemp + Para_Gen3_Type_Offset);			

}



/************************************************************************/
/* 	Write Line 1 and Line 2 of LCD										*/
/************************************************************************/
void Update_LCD(void)
{

/* Set Line 1 Address */
Write_Control_LCD_CSEL;
PORTD = LCD_Line_1;
Write_Control_Enable_LCD_CSEL;
DelayUs(50);
Write_Control_LCD_CSEL;

DelayMs(2);

/*set the pointers at the line array adddresses */
line1_ptr = &(line1[0]);
for (loop = 0; loop < 16; loop++)
	{
		/* Write string 1 to LCD */
		Write_Data_Enable_LCD_CSEL;
		PORTD = *line1_ptr;
		Write_Data_LCD_CSEL;
		DelayUs(50);
		Write_Data_Enable_LCD_CSEL;
		line1_ptr++;
	}


/* Set Line 2 Address */
Write_Control_LCD_CSEL;
PORTD = LCD_Line_2;
Write_Control_Enable_LCD_CSEL;
DelayUs(50);
Write_Control_LCD_CSEL;
DelayMs(4);

/*set the pointers at the line array adddresses */
line2_ptr = &(line2[0]);
for (loop = 0; loop < 16; loop++)
	{
		/* Write string 2 to LCD */
		Write_Data_Enable_LCD_CSEL;
		PORTD = *line2_ptr;
		Write_Data_LCD_CSEL;
		DelayUs(50);
		Write_Data_Enable_LCD_CSEL;
		line2_ptr++;
	}	
	
}


/************************************************************************/
/* 						Init  											*/
/*  This is for a standard 16 x 2 character LCD... 						*/
/************************************************************************/
void Init_LCD(void)
{
/* Function Select */
Write_Control_LCD_CSEL;
PORTD = LCD_Function;
Write_Control_Enable_LCD_CSEL;
DelayUs(5);
Write_Control_LCD_CSEL;

DelayUs(40);

/* Display On */
PORTD = LCD_Display_On;
Write_Control_Enable_LCD_CSEL;
DelayUs(5);
Write_Control_LCD_CSEL;

DelayUs(40);

/* Display Clear */
PORTD = LCD_Display_Clear;
Write_Control_Enable_LCD_CSEL;
DelayUs(5);
Write_Control_LCD_CSEL;

DelayMs(2);

/* Entry Mode */
PORTD = LCD_Display_Entry_Mode;
Write_Control_Enable_LCD_CSEL;
DelayUs(5);
Write_Control_LCD_CSEL;

/* Display Clear */
Write_Control_LCD_CSEL;
PORTD = LCD_Display_Clear;
Write_Control_Enable_LCD_CSEL;
DelayUs(50);
Write_Control_LCD_CSEL;

DelayMs(2);
}




/************************************************************************/
/* 						Display relevant data on the LCD 			
Need to build display lies for:
Display_Fn = 
	Choosing_Fn	
	Volume_Mode
	XO_Mode 
	Save_Mode
	Load_Mode
	EQ_Mode
	Change_Mode															*/
/*
and the following explicit functions
	XO_Band_Sel_Fn
	XO_Parm_Sel
	XO_LF_Parm_Fn
	XO_HF_Parm_Fn
	XO_SL_Parm_Fn
	XO_SU_Parm_Fn
	XO_Del_Parm_Fn
	XO_Att_Parm_Fn

	Save_Sel_Fn

	Load_Sel_Fn															*/

/*
#define EQ_Sel_Fn 				
#define EQ_Freq_Sel_Fn 			
#define EQ_Q_Sel_Fn  	
#define EQ_Gain_Sel_Fn 	
#define EQ_Type_Sel_Fn 	
																		*/
/************************************************************************/
void Display_Data(char Display_Band, char Display_Fn, int Display_Value)
{
char loop;
int Display_Value_Temp;


/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;

/*set the pointers at the line array adddresses */
/* The idea here is to use pointers into the Line1 and Line 2 arrays */
/* The code efficienct using direct array operations is abysmal.     */
/* Remember that this fuction builds the lines from right to left    */
/* You did the because it makes it easer to build the numbers....    */
/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;


/********************************************************************/
/* 					Build Line 1 string in here 					*/
/********************************************************************/

/************************************************************************/
/* Choosing Value - user selecting the fuinction to modify 				*/
/************************************************************************/
	if (Display_Fn == Choosing_Fn)
		{
		*line1_ptr = Choose_Str[loop];
		*line2_ptr = Choice_Display[Display_Value - 1][loop];					
		}


/************************************************************************/
/* Volume Function - user modifying the volume 							*/
/************************************************************************/
	else if (Display_Fn == Vol_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = Vol_Str[loop];
		*line2_ptr = Vol2_Str[loop];
		/* Convert integer to ASCII */
		/* Put volume in last three characters of line 1 */
		if (loop == 12)
			{
			display_temp = Display_Value % 2;
			/* is there a 0.5dB step? */
			if (display_temp == 1)
				{
				*line1_ptr = (5)+48;  /* 48 is offset of "0" ASCII */
				}
			else
				{
				*line1_ptr = (0)+48;  /* 48 is offset of "0" ASCII */
				}
			Display_Value = Display_Value/2;
			}
		if (loop == 11)
			{
				*line1_ptr = 46;  /* 46 is "." */
			}
		if ((loop <11) && (loop > 6))
			{
			if ((display_temp != 0) && (Display_Value == 0))
				{
				*line1_ptr = 45; 	 				/* 45 is offset of "-" ASCII */
				}
			display_temp = Display_Value % 10;
			/* DISPLAY ZERO on leading character only if it actually is ZERO! */
			if((Display_Value == 0) && (loop == 10))
				{
				*line1_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
				if (*(line1_ptr + 2) ==  (5+48))
					{
					display_temp = 1;				/* do this to get a - sign on next character... */
					}
				}
			else if (Display_Value > 0)
				{
				*line1_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
				Display_Value = Display_Value/10;
				}
			}
		}

/************************************************************************/
/* Crossover band Select 					 							*/
/************************************************************************/
	else if (Display_Fn == XO_Band_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = XO_Str[loop];

		/* Convert 0-3 integer to 1-4 ASCII for band number */
		/* tweak this to make bands come up as 1 .. 4 */
		if (loop == 14) 
			*line1_ptr = Display_Value + 49;
		}


/************************************************************************/
/* Crossover Parameter Selection for user to change						*/
/************************************************************************/
	else if (Display_Fn == XO_Parm_Sel)
		{
		/* Build string */
		*line1_ptr = XO_Choice_Display[(Display_Value - XO_LF_Parm_Fn) + 1][loop];
		*line2_ptr = Sel_Str[loop];
		/* Convert 0-3 integer to 1-4 ASCII for band number */
		/* tweak this to make bands come up as 1 .. 4 */
		if (loop == 5) 
			*line1_ptr = Display_Band + 49;			
		}

/************************************************************************/
/*Crossover Parameter modification display								*/
/************************************************************************/
	else if ((Display_Fn >= XO_LF_Parm_Fn) && (Display_Fn <= XO_Inv_Parm_Fn))
		{
		/* Build string */
		*line1_ptr = XO_Choice_Display[(Display_Fn - XO_LF_Parm_Fn)+1][loop];

		/* Convert 0-3 integer to 1-4 ASCII for band number */
		/* tweak this to make bands come up as 1 .. 4 */
		if (loop == 5) 
			*line1_ptr = Display_Band + 49;		

		/* For displays requiring numerical output, Convert integer to ASCII */
		/* except attenuation - this is in 0.5dB steps						*/
		if ((Display_Fn != XO_Att_Parm_Fn)&& (Display_Fn != XO_SL_Parm_Fn) && (Display_Fn != XO_SU_Parm_Fn)&& (Display_Fn != XO_Inv_Parm_Fn))
			{
			if (loop <= 4)
				{
				display_temp = Display_Value % 10;
				if ((Display_Value > 0) || (loop == 3))
					*line2_ptr = (display_temp)+48;
				else
					*line2_ptr = Blank_Char;
		
				Display_Value = Display_Value/10;
				}
			/* put up units display for crossover parameters */
			if ((loop >= 6) && (loop < 11))
				*line2_ptr = XO_Units_Display[Display_Fn - XO_LF_Parm_Fn][loop-6];
			}

		/* For attenyation display - need numerical output, Convert integer to 0.5 dB steps ASCII */
		if (Display_Fn == XO_Att_Parm_Fn)
			{
			if (loop == 4)
				{
				display_temp = Display_Value % 2;	
				/* is there a 0.5dB step? */
				if (display_temp == 1)
					{
					*line2_ptr = (5)+48;  /* 48 is offset of "0" ASCII */
					}
				else
					{
					*line2_ptr = (0)+48;  /* 48 is offset of "0" ASCII */
					}
				Display_Value = Display_Value/2;
				}
			if (loop == 3)
				{
					*line2_ptr = 46;  /* 46 is "." */
				}
			if (loop < 3)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && (loop == 2))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}

			/* put up units display for crossover parameters */
			if ((loop >= 6) && (loop < 11))
				*line2_ptr = XO_Units_Display[Display_Fn - XO_LF_Parm_Fn][loop-6];
			}


			
		/* Else for displays requiring string put up const string...
		Now if the display is of the XO slope, put in the right string */
		if ((Display_Fn == XO_SL_Parm_Fn) || (Display_Fn == XO_SU_Parm_Fn))				
			{
			if (loop <= 3)
				*line2_ptr = Slopes[Display_Value][loop];
			}

		else if (Display_Fn == XO_Inv_Parm_Fn)
			{
			if (loop <= 2)
				{
				if (Display_Value == Invert_True)
					*line2_ptr = Yes_No[0][loop];
				else if (Display_Value == Invert_False)
					*line2_ptr = Yes_No[1][loop];
				}				
			}
		}


/************************************************************************/
/* Equaliser Selection - Band 1-4 and Common 1,2						*/
/************************************************************************/
	else if (Display_Fn == EQ_Sel_Fn)
		{
		/* Build string */
		if (loop < 9)
			*line1_ptr = EQ_Str[loop];
		else 
			{
			if (Display_Value < Gen1_Para_data)    /*all the band eq's occur below this value */
				{
				*line1_ptr = EQ_Choice_Display[0][loop - 9];
				/* OK, this is a magic number - there are 3 para EQs per channel... */
				if (loop == 14) *line1_ptr = (Display_Value/3)+49;
				/* OK, this is a magic number again - there are still 3 para EQs per channel... */
				if ((loop == 15) && ((Display_Value % 3)==0)) *line1_ptr = 'a';
				else if ((loop == 15) && ((Display_Value % 3) == 1)) *line1_ptr = 'b';
				else if ((loop == 15) && ((Display_Value % 3) == 2)) *line1_ptr = 'c';
				}
			else
				{
				*line1_ptr = EQ_Choice_Display[1][loop - 9];
				if (loop == 14) *line1_ptr = (Display_Value - Gen1_Para_data)+49;
				if (loop == 15) *line1_ptr = Blank_Char;
				}
			}
		}


/************************************************************************/
/* Display EQualiser parameters, freq, Q, Gain and Type					*/
/************************************************************************/
	else if ((Display_Fn == EQ_Freq_Sel_Fn) || (Display_Fn == EQ_Q_Sel_Fn) || 
			 (Display_Fn == EQ_Gain_Sel_Fn) || (Display_Fn == EQ_Type_Sel_Fn))
		{
			/* Build string */
			if (loop < 5)
			{
				if (Display_Band < Gen1_Para_data)
					*line1_ptr = EQ_Choice_Display[0][loop];
				else
					*line1_ptr = EQ_Choice_Display[1][loop];
			}
			else
			*line1_ptr = EQ_Select_Display[Display_Fn - EQ_Freq_Sel_Fn][loop - 5];


			
			if (loop == 5)
			{
				/* OK, this is a magic number - there are 3 para EQs per channel... */
				if (Display_Band < Gen1_Para_data)    /* then it is one of the band specific EQ's */
					{
					*line1_ptr = (Display_Band/3) + 49;
					}
				else
					{
					*line1_ptr = (Display_Band + 49 - Gen1_Para_data);    /* then it is one of the common EQ's, which start here */
					}
			} 
			if (loop == 6)  /* then we need to identify which of the band specific EQ'w it is */
			{
 				/* OK, this is a magic number - there are 3 para EQs per channel... */
				if (Display_Band < Gen1_Para_data)    /* then it is one of the band specific EQ's */
					{
					if ((Display_Band % 3)==0) *line1_ptr = 'a';
					else if ((Display_Band % 3) == 1) *line1_ptr = 'b';
					else if ((Display_Band % 3) == 2) *line1_ptr = 'c';
					}
				else
					{
					*line1_ptr = Blank_Char;         /* then it is one of the common EQ's, which start here */
					}
			} 
			
			if ((Display_Fn == EQ_Type_Sel_Fn) && (loop < 10))
				*line2_ptr = EQ_Type_Display[Display_Value][loop];
			
			if (Display_Value < 0)
				Display_Value = - Display_Value;

			if (Display_Fn != EQ_Type_Sel_Fn)
				if (loop < 5)
				{
					if((loop == 3) && (Display_Fn == EQ_Q_Sel_Fn))
						*line2_ptr = 46;
					else
					{
						display_temp = Display_Value % 10;
						if ((Display_Value > 0) || (loop == 4))
							*line2_ptr = (display_temp)+48;
						else
							*line2_ptr = Blank_Char;
						Display_Value = Display_Value/10;
					}
				}
				else if ((loop > 5) && (loop < 11))
				{
					*line2_ptr = EQ_Units_Display[Display_Fn - EQ_Freq_Sel_Fn][loop - 6];
				}
				
			if ((Display_Fn == EQ_Gain_Sel_Fn) && (loop == 2) && (Display_Value_Temp < 0))
				*line2_ptr = 45;   /* - sign */
		}

/************************************************************************/
/* Save Function display												*/
/************************************************************************/
	else if (Display_Fn == Save_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = Save_Str[loop];
	
		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (loop == 13) 
			*line1_ptr = Display_Value + 49;
		}

/************************************************************************/
/*Load function display													*/
/************************************************************************/
	else if (Display_Fn == Load_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = Load_Str[loop];
			
		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (loop == 15) 
			*line1_ptr = Display_Value + 49;
		}


/************************************************************************/
/* Input Selection Function display										*/
/************************************************************************/
	else if (Display_Fn == In_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = Input_Str[loop];
	
		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (Display_Value == (int)Input_SPDIF) 
			*line2_ptr = Input_Choices[0][loop];
		else if (Display_Value == (int)Input_ADC) 
			*line2_ptr = Input_Choices[1][loop];
		}


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 													*/
	/************************************************************************/


Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}



//Print_Data(int Number_of_Words, char *ptr_to_Data)
void Print_Data(int Number_of_Words, unsigned char *ptr_to_Data, unsigned int Address_to_Print)
{
char loop;
unsigned char display_temp, display_data;
unsigned int Address_Temp, Address_Temp_Hex;

	/*set the pointers at the line array adddresses */
	/* The idea here is to use pointers into the Line1 and Line 2 arrays */
	/* The code efficiency using direct array operations is abysmal.     */
	/* Remember that this fuction builds the lines from right to left    */
	/* You did the because it makes it easer to build the numbers....    */
	/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
display_data = *ptr_to_Data;    /* initial dtaa to diaplay */
Address_Temp = Address_to_Print;
Address_Temp_Hex = Address_to_Print;

for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = display_data % 16;
	if (loop > (15 - 2*Number_of_Words))
		{
			if (display_temp <= (unsigned char)9)
				*line1_ptr = (display_temp)+48;
			else
				*line1_ptr = (display_temp)+55;
		}
	display_data = display_data>>4;
	if (!(loop % 2)  && (((15 - loop)/2) < Number_of_Words))  /* then need to read next byte */
		{
		ptr_to_Data++;
		display_data = *ptr_to_Data;
		}

	display_temp = Address_Temp % 10;
	if ((Address_Temp > (unsigned int)0) || (loop > 14))
		*line2_ptr = (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Address_Temp = Address_Temp/10;


	if ((loop <= 8) && (loop > 4))
		{
			display_temp = Address_Temp_Hex % 16;
			if (display_temp <= (unsigned char)9)
				*line2_ptr = (display_temp)+48;
			else
				*line2_ptr = (display_temp)+55;
			Address_Temp_Hex = Address_Temp_Hex>>4;
		}


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 													*/
	/************************************************************************/


Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}







void Display_Debug_Data(int Display_Value)
{
char loop;
int Display_Value_Temp;
		/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;
		/*set the pointers at the line array adddresses */
		/* The idea here is to use pointers into the Line1 and Line 2 arrays */
		/* The code efficienct using direct array operations is abysmal.     */
		/* Remember that this fuction builds the lines from right to left    */
		/* You did the because it makes it easer to build the numbers....    */
		/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = Display_Value % 16;
	if ((Display_Value_Temp > 0) || (loop > 11))
		{
			if (display_temp <= 9)
				*line1_ptr = (display_temp)+48;
			else
				*line1_ptr = (display_temp)+65;
		}
	else
		*line1_ptr = Blank_Char;
	Display_Value_Temp = Display_Value_Temp/16;


	display_temp = Display_Value % 10;
	if ((Display_Value > 0) || (loop < 11))
		*line2_ptr = (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Display_Value = Display_Value/10;


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 													*/
	/************************************************************************/


Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


void Display_Debug_Data_Char(unsigned char Display_Value)
{
char loop;
int Display_Value_Temp;


/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;

/*set the pointers at the line array adddresses */
/* The idea here is to use pointers into the Line1 and Line 2 arrays */
/* The code efficienct using direct array operations is abysmal.     */
/* Remember that this fuction builds the lines from right to left    */
/* You did the because it makes it easer to build the numbers....    */
/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = (unsigned char) Display_Value_Temp % 16;
	if ((Display_Value_Temp > 0) || (loop > 13))
		{
			if (display_temp <= 9)
				*line1_ptr = (unsigned char) (display_temp)+48;
			else
				*line1_ptr = (unsigned char) (display_temp)+55;
		}
	else
		*line1_ptr = Blank_Char;
	Display_Value_Temp = (unsigned char) Display_Value_Temp/16;


	display_temp = (unsigned char)Display_Value % 10;
	if ((Display_Value > (unsigned char)0) || (loop > (char)13))
		*line2_ptr = (unsigned char) (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Display_Value = (unsigned char) Display_Value/10;


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 													*/
	/************************************************************************/


Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}







/* Call this to completely update all filter settings etc */
void Update_Filter_Set(unsigned int Sample_Rate)
{
char local_loop;	
	for (local_loop = 0; local_loop < 4; local_loop++)
	{
		Temp_Data_Values_Pointer = &Data_Values[local_loop];
	    Write_Filter_Parm(0x10 + local_loop, Temp_Data_Values_Pointer->Fl, Temp_Data_Values_Pointer->Sl, Sample_Rate,0 ,0); 
	    Write_Filter_Parm(0x00 + local_loop, Temp_Data_Values_Pointer->Fu, Temp_Data_Values_Pointer->Su, Sample_Rate,0 ,0); 
		Temp_EQ_Values_Pointer = &EQ_Values[local_loop];
	    Write_Filter_Parm(       local_loop, Temp_EQ_Values_Pointer->CF, (Temp_EQ_Values_Pointer->Type) + 0x10, Sample_Rate,Temp_EQ_Values_Pointer->Q ,Temp_EQ_Values_Pointer->Gain); 
	}
	Temp_EQ_Values_Pointer = &EQ_Values[Gen1_Para_data];
    Write_Filter_Parm(       Gen1_Para_data, Temp_EQ_Values_Pointer->CF, (Temp_EQ_Values_Pointer->Type) + 0x10, Sample_Rate,Temp_EQ_Values_Pointer->Q ,Temp_EQ_Values_Pointer->Gain); 
	Temp_EQ_Values_Pointer = &EQ_Values[Gen2_Para_data];
    Write_Filter_Parm(       Gen2_Para_data, Temp_EQ_Values_Pointer->CF, (Temp_EQ_Values_Pointer->Type) + 0x10, Sample_Rate,Temp_EQ_Values_Pointer->Q ,Temp_EQ_Values_Pointer->Gain); 
	Temp_EQ_Values_Pointer = &EQ_Values[Gen3_Para_data];
    Write_Filter_Parm(       Gen3_Para_data, Temp_EQ_Values_Pointer->CF, (Temp_EQ_Values_Pointer->Type) + 0x10, Sample_Rate,Temp_EQ_Values_Pointer->Q ,Temp_EQ_Values_Pointer->Gain); 
}


/* Call this to update all Atten settings etc 									*/
/*	inputs																		*/
/*		DAC_Atten - attenution sought in 0.5dfB steps							*/
/*		Data_Values (static variable) with band attenuation in 0.5dB steps		*/
/*	Outputs																		*/
/*		None																	*/
/*	Calls Write Atten Loop with required attentaion in o.5dB steps				*/
void Update_Atten_Set(unsigned char DAC_Atten)
{
int temp_atten;
	/* first handle the gain deltas for each channel - do this in the ADAU1442 */
	for (loop = 0; loop < 4; loop++)
	{
		temp_atten = (unsigned int)Data_Values[loop].Atten;
		if (temp_atten <  Volume_Atten_Min)
			temp_atten = (unsigned int)Volume_Atten_Min;
		if (temp_atten > Volume_Atten_Max)
			temp_atten = (unsigned int)Volume_Atten_Max;
		Write_Atten(loop, (unsigned char) temp_atten);	
	}

temp_atten = (unsigned int)Volume_Atten_Max - (unsigned int)DAC_Atten;
	if (temp_atten <  Volume_Atten_Min)
		temp_atten = (unsigned int)Volume_Atten_Min;
	if (temp_atten > Volume_Atten_Max)
		temp_atten = (unsigned int)Volume_Atten_Max;
	/* now write attenuation for the PGA2320 devices  */
	Write_PGA2320((unsigned char)temp_atten);
}


/* Call this to update all Atten settings etc */
void Update_Delay_Set(unsigned int Sample_Rate)
{
	for (loop = 0; loop < 4; loop++)
	{
		Write_Delay(loop, Data_Values[loop].Delay_mm, Sample_Rate);	
	}
}




#pragma code highvector=0x000008
void highvector (void)
{
	_asm GOTO timer_isr _endasm
}
#pragma code

/****************************************************************************/	
/* Uses the following globals 												*/
/* s_count 																	*/
/* Old_Rot_Status    Records state of rotary control 						*/
/* Fast_Loop_Counter  Used to set Fast_Cnt 									*/
/* Slow_Loop_Counter  Used to see if user turning control continuously 		*/
/*																			*/	
/*																			*/	
/* This sets is the ISR that deals with key presses etc						*/
/****************************************************************************/	
#pragma interrupt timer_isr
void 
timer_isr (void)
{
	unsigned char Current_Rot_Status = 0;

	INTCONbits.GIE = 0;
	Current_Rot_Status = PORTA & (Vals_0 + Vals_1 + Exit + Sel);  /*Read Rot Status and mask off bits of interest */

	Key_Just_Pressed = (Current_Rot_Status != Old_Rot_Status);

	/* Only do this if something has changed */
	if (Key_Just_Pressed  && !(Current_Rot_Status & (Sel + Exit)))
	{
		if (Old_Rot_Status == Phase_0)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_1)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;
		}
		else if (Old_Rot_Status == Phase_1)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_2)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;			
		}
		else if (Old_Rot_Status == Phase_2)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_3)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;
		}
		else if (Old_Rot_Status == Phase_3)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_0)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;			
		}
	
		Slow_Loop_Counter = Slow_Counter_Init;	/* If key just pressed initialise slow counter */
	
		if (Fast_Loop_Counter > (unsigned int)0)
			Fast_Loop_Counter--;
	}
	else if (Key_Just_Pressed && (Current_Rot_Status & (Sel + Exit)))
	{
		if (Current_Rot_Status & Sel)
			Keypress_Read = Sel;			
		if (Current_Rot_Status & Exit)
			Keypress_Read = Exit;			
		Fast_Change = 1;
	}		
	else    /* else not Key_just_Pressed */
	{
		Keypress_Read = 0;
		if (Slow_Loop_Counter > (unsigned int)0)
			Slow_Loop_Counter--;
	}


	if (Slow_Loop_Counter == (unsigned int)0)
	{
		Fast_Change = 1;
		Fast_Loop_Counter = Fast_Counter_Init;
	}

	if (Fast_Loop_Counter == (unsigned int)0)   /* move into fast change of value for user */
	{
		Fast_Change = 10;
	}
			
	Old_Rot_Status = Current_Rot_Status;   /*Record Rot Status to allow comparison next ISR loop */
	INTCONbits.TMR0IF = 0;
	INTCONbits.GIE = 1;
}


void main(void) {
int BandTemp; 
char Update_Display = 1; 			/* Flag saying need to update display */
unsigned int Temp;   				/* define this globally as we can save stome storage this way */
unsigned int Sample_Rate = 48000; 	/* CD = 44100, Analog = 48000 */
unsigned char DAC_Atten = 0;      	/* literally the attenuation for the DAC, i.e. Volume Control */
unsigned char Input_Selected = Input_ADC;   /* Selects between inputs 0 = SPDIF, 1 = ADC */
char Auto_Repeat = 0;             	/* use this to keep track of whether a Auto Repeat button Press */
long int Auto_Revert = Auto_Revert_Init_Val;	
									/* Use this as a counter - after 255 loops auto revert to volume mode */
char Band = 0;           			/* use to drive state machine band select */
int Step_Size = 0;        			/* Use this to implement push button steps */  
int Display_Value = Volume_Mode;  	/* use this to display the current value */
char Mode = Volume_Mode;			/* use this to track current mode */
char Memory_Bank = Default_Mem_Bank;/* use this to track which memory bank is in use */
char Sel_Fn = Choosing_Fn;			/* Use this to track current Selection */
char XO_Parm = XO_LF_Parm_Fn;		/* Default to XO_LF_Parm_Fn */
char EQ_Parm = Gen1_Para_data;		/* Default to General 1 parametric EQ */

/*Allows system to settle*/
DelayMs(200);

/* intitialise the Micro I/O ports*/
Init_IO();

/*Allows ADAU1442 to settle*/
DelayMs(200);
Init_LCD();					/* initialise the LCD */
DelayMs(200);
DelayMs(200);
Update_LCD();				/* Writes Line1 and Line2 to LCD */
DelayMs(200);
DelayMs(200);

/* Initialise and load parameters from EEPROM - we need these to load defaults & saved values */
Load_From_Memory(Memory_Bank); /* Read Data from ROM */
DAC_Atten = EEPROM_READ(Volume_Offset);   /* Volume */
Input_Selected = EEPROM_READ(Input_Selected_Offset);   /* Input Selected from EEPROM */
Band = 0;				/* set this to a sensible value */
Data_Values_Pointer = &(Data_Values[Band]);
EQ_Values_Pointer = &(EQ_Values[Band]);

/* Now iitialise the DSP and programmable paramaters per defaults and saved values*/
Init_DSP();											/* Load Code into DSP  */
	/* This stuff initialises some registers programmable elements that need config on powerup */
//Write_Input(Input_Selected);						/* Set Input */
Init_ADC();											/* clears and sets up the ADC and DAC */
Update_Atten_Set(0);
Write_Input(Input_Selected);

/* checked to here */

/* need to sort out comm filt 0-->  */
/* mute output till the filters are right!!!*/
Update_Atten_Set(Mute);
Write_DCBlock();									/* Load DC Block Setting */
Update_Filter_Set(Sample_Rate);						/* Load Filter Settings */
Update_Delay_Set(Sample_Rate);						/* Load Delay Settings */
Display_Data(Band, Sel_Fn, Display_Value);
Update_Atten_Set(DAC_Atten);									/* Load Attenuator Settings including volume */

/****************************************************************************/	
/* Get reasonable initialisation, go straight to volume						*/
/****************************************************************************/	
Band = 0;
Sel_Fn = Vol_Sel_Fn;
Display_Value = DAC_Atten;
Display_Data(Band, Sel_Fn, Display_Value);


/*
State Machine that looks after user input

This will be an infinite loop that:
	- Looks at inputs
	- If one has changed, acts on it by
		- If band change input, updates band being changed.
			- Update Band_No
		- If Fn Select, updates funtion being changed 
			- Update Fn_Sel.  
		- If Value, then change value of Band_Sel -->> Fn_Sel, i.e. Band1 -->> B1L
		- If Save, writes current Band_No --> Fn_Sel value to EEPROM
		- If Volume
			- Changes Volume value
			- write value to EEPROM
	- Calls Display function, that once per cycle:
		- Displays Band_No
		- Displays Fn_Sel
		- Displays Val3
		- Displays Val2
		- Displays Val1
		- Displays Val0
*/


/****************************************************************************/	
/* This sets up the test ISR 												*/
/* Should be about 500us cycle 												*/
/****************************************************************************/	
OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_8BIT & T0_PS_1_16);
INTCONbits.TMR0IE = 1; // Enable Timer0
INTCONbits.RBIE = 0;
INTCONbits.TMR0IF = 0;
INTCONbits.INT0IF = 0;
INTCONbits.RBIF = 0;

//INTCONbits.PEIE = 1; // Enable Peripheral interrupt
INTCONbits.GIE = 1; // Enble Global interrupt 
//Nop();

/* give things a bit to settle */
DelayMs(100);
Step_Size = 0;
Key_Just_Pressed = 0;
DelayMs(100);
/* 
Buttons are: Mode_Sel, Fn_Sel, Vol_Dn, Vol_Up, Exit, Vals_0, Vals_1, Vals_2 
Watch what is done with them, and act as required
Band, Fn, Val
*/
do
{
/* Look after changing modes */
	if (Key_Just_Pressed)
	{
		Update_Display = 1;
		Auto_Revert = Auto_Revert_Init_Val;		/* If key pressed, reset the counter */

		/********************************************************************/
		/* Currently in Change Mode - act on legal button presses 			*/
		/********************************************************************/
		if (Sel_Fn == Choosing_Fn)
		{
			/* Up or Down*/
			if (Up_Keys || Down_Keys)
			{
				if (Up_Keys)
					Mode += 1;
				else
					Mode -= 1;
				/* check for wrap around */
				if (Mode > In_Sel_Mode)
					Mode = Volume_Mode;
				if (Mode < Volume_Mode)
					Mode = In_Sel_Mode;
			}

			/* Select Key hit */
			else if Sel_Key
			{
				Sel_Fn = Mode*10;
			}
			/* Exit */
			else if Exit_Key
			{
				Sel_Fn = Choosing_Fn;
			}
			
			else;
			
			if (Sel_Fn == Choosing_Fn)
				Display_Value = Mode;
			else if (Sel_Fn == Vol_Sel_Fn)
				Display_Value = DAC_Atten;
			else if (Sel_Fn == XO_Band_Sel_Fn)
				Display_Value = Band;
			else if (Sel_Fn == Save_Sel_Fn)
				Display_Value = Memory_Bank;
			else if (Sel_Fn == Load_Sel_Fn)
				Display_Value = Memory_Bank;
			else if (Sel_Fn == EQ_Sel_Fn)
				{
					Display_Value = Band_1_Para_data;  /* this is band 1 */
					EQ_Parm = Band_1_Para_data;
					Band = EQ_Parm;
				}
			else if (Sel_Fn == In_Sel_Fn)
				Display_Value = Input_Selected;
			else;

		}


		/********************************************************************/
		/* Currently in Volume Select Function - act on legal button presses*/
		/********************************************************************/
		else if (Sel_Fn == Vol_Sel_Fn)
		{
			/* Up */
			if Down_Keys
			{
				if (DAC_Atten <= (unsigned char) (Volume_Atten_Max - Fast_Change)) 
				{
					DAC_Atten = DAC_Atten + Fast_Change;
					Update_Atten_Set(DAC_Atten);
 				}	
				Display_Value = DAC_Atten;
			}
			/* Down */
			else if Up_Keys
			{
				if (DAC_Atten >= (unsigned char)(Volume_Atten_Min + Fast_Change)) 
				{
					DAC_Atten = DAC_Atten - Fast_Change;
					Update_Atten_Set(DAC_Atten);
 				}
				Display_Value = DAC_Atten;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				EEPROM_WRITE(Volume_Offset, DAC_Atten);
				Display_Value = Mode;
			}
			else;
		}

		/************************************************************************/
		/* Currently in XO mode - look after key presses and function changes 	*/
		/************************************************************************/
		else if(Sel_Fn == XO_Band_Sel_Fn)
		{
			/*	This section works out which band the user wants to set 
				XO parameters for 											*/
			/* Up */
			if Up_Keys
			{
				if (Band < Max_Bands)
					Band += 1;
				else
					Band = 0;   /* always start from band zero */
				Display_Value = Band;
			}
			/* Down */
			else if Down_Keys
			{
				if (Band > 0)
					Band -= 1;
				else
					Band = Max_Bands;
				Display_Value = Band;
		}
			/* Select Key hit */
			else if Sel_Key
			{
				Sel_Fn = XO_Parm_Sel;
				XO_Parm = XO_LF_Parm_Fn;		/* Start at LF Parms */
				Display_Value = XO_Parm;
			}
			/* Exit */
			else if Exit_Key
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
			/* Use pointer to Data_Values array to save code space 				*/
			/* Just don't go changing Band without updating this pointer!!! 	*/
			Data_Values_Pointer = &(Data_Values[Band]);
		}
			
		/************************************************************************/
		/* Currently in XO Parameter Select mode - look after key presses 
			and function changes 												*/
		/************************************************************************/
		else if(Sel_Fn == XO_Parm_Sel)
		{
			/*	This section works out which Parameter the user wants to set	*/ 
			/*	XO parameters for 												*/
			/* Up 																*/
			if Up_Keys
			{
				if (XO_Parm < XO_Att_Parm_Fn)
					XO_Parm += 1;
				else
					XO_Parm = XO_LF_Parm_Fn;   /* always start from XO_LF_Parm_Fn */
				Display_Value = XO_Parm;
			}
			/* Down */
			else if Down_Keys
			{
				if (XO_Parm > XO_LF_Parm_Fn)
					XO_Parm -= 1;
				else
					XO_Parm = XO_Att_Parm_Fn;	/* Jump it back up to Att Parm Fn */
				Display_Value = XO_Parm;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Sel_Fn = XO_Parm;
				if(Sel_Fn == XO_LF_Parm_Fn)
					Display_Value = Data_Values_Pointer->Fl;
				else if(Sel_Fn == XO_HF_Parm_Fn)
					Display_Value = Data_Values_Pointer->Fu;
				else if (Sel_Fn == XO_SL_Parm_Fn)
					Display_Value = Data_Values_Pointer->Sl;
				else if (Sel_Fn == XO_SU_Parm_Fn) 
					Display_Value = Data_Values_Pointer->Su;
				else if (Sel_Fn == XO_Del_Parm_Fn) 
					Display_Value =  Data_Values_Pointer->Delay_mm;
				else if (Sel_Fn == XO_Att_Parm_Fn) 
					Display_Value =  Data_Values_Pointer->Atten;
				else;
				}
			/* Exit */
			else if Exit_Key
			{
				Sel_Fn = XO_Band_Sel_Fn;
				Display_Value = Band;

			}
		}

		/************************************************************************/
		/* Low Frequency Crossover Parameters									*/
		/************************************************************************/
		else if(Sel_Fn == XO_LF_Parm_Fn)
		{
			/*	This section reads user input for low frequency 				*/
			/*		XO parameters for 											*/
			/* Up 																*/
			if Up_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = Fast_Change_Frequencies;
				else
					Step_Size = 10;
				Data_Values_Pointer->Fl = Data_Values_Pointer->Fl + Step_Size;
				if (Data_Values_Pointer->Fl > Fl_Max) Data_Values_Pointer->Fl = Fl_Max;
    			Write_Filter_Parm(0x10 + Band, Data_Values_Pointer->Fl, Data_Values_Pointer->Sl, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Fl;
			}
			/* Down */
			else if Down_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = -Fast_Change_Frequencies;
				else
					Step_Size = -10;
				Data_Values_Pointer->Fl = Data_Values_Pointer->Fl + Step_Size;
				if (Data_Values_Pointer->Fl < Fl_Min) Data_Values_Pointer->Fl = Fl_Min;
		 		Write_Filter_Parm(0x10 + Band, Data_Values_Pointer->Fl, Data_Values_Pointer->Sl, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Fl;
		}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_HF_Parm_Fn;
				Display_Value = Data_Values_Pointer->Fu;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}
			else;
			

			if Exit_Key
				Display_Value = XO_Parm;
			else;

		}
		

		/************************************************************************/
		/* High Frequency Crossover Parameters									*/
		/************************************************************************/
		else if(Sel_Fn == XO_HF_Parm_Fn)
		{
			/*		This section reads user input for High frequency 			*/
			/*		XO parameters for 											*/
			/* Up 																*/
			if Up_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = Fast_Change_Frequencies;
				else
					Step_Size = 10;
				Data_Values_Pointer->Fu = Data_Values_Pointer->Fu + Step_Size;
				if (Data_Values_Pointer->Fu > Fu_Max) Data_Values_Pointer->Fu = Fu_Max;
			    Write_Filter_Parm(0x00 + Band, Data_Values_Pointer->Fu, Data_Values_Pointer->Su, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Fu;
			}
			/* Down */
			else if Down_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = -Fast_Change_Frequencies;
				else
					Step_Size = -10;
				Data_Values_Pointer->Fu = Data_Values_Pointer->Fu + Step_Size;
				if (Data_Values_Pointer->Fu < Fu_Min) Data_Values_Pointer->Fu = Fu_Min;
			    Write_Filter_Parm(0x00 + Band, Data_Values_Pointer->Fu, Data_Values_Pointer->Su, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Fu;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_SL_Parm_Fn;
				Display_Value = Data_Values_Pointer->Sl;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}

			if Exit_Key
				Display_Value = XO_Parm;
			else;

		}
		

		/************************************************************************/
		/* Low Frequency Crossover Slope										*/
		/************************************************************************/
		else if(Sel_Fn == XO_SL_Parm_Fn)
		{
			/*		This section reads user input for low frequency XO slope	*/
			/*		XO parameters for 											*/
			/* Up 																*/
			if Up_Keys
			{
				Step_Size = 1;
				Data_Values[Band].Sl += Step_Size;
				if (Data_Values_Pointer->Sl > Sl_Max) Data_Values_Pointer->Sl = Sl_Min;
   				Write_Filter_Parm(0x10 + Band, Data_Values_Pointer->Fl, Data_Values_Pointer->Sl, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Sl;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = 1;
				if (Data_Values_Pointer->Sl > Sl_Min) Data_Values_Pointer->Sl -= 1;
				else Data_Values_Pointer->Sl = Sl_Max;
				Write_Filter_Parm(0x10 + Band, Data_Values_Pointer->Fl, Data_Values_Pointer->Sl, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Sl;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_SU_Parm_Fn;
				Display_Value = Data_Values_Pointer->Su;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}

			if Exit_Key
				Display_Value = XO_Parm;
			else;
		}
		

		/************************************************************************/
		/* High Frequency Crossover Slope										*/
		/************************************************************************/
		else if(Sel_Fn == XO_SU_Parm_Fn)
		{
			/*		This section reads user input for high frequency XO slope	*/
			/*		XO parameters for 											*/
			/* Up 																*/
			if Up_Keys
			{
				Step_Size = 1;
				Data_Values_Pointer->Su += Step_Size;
				if (Data_Values_Pointer->Su > Su_Max) Data_Values_Pointer->Su = Su_Min;
			    Write_Filter_Parm(0x00 + Band, Data_Values_Pointer->Fu, Data_Values_Pointer->Su, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Su;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = 1;
				if (Data_Values_Pointer->Su > Su_Min) Data_Values_Pointer->Su -= 1;
				else Data_Values_Pointer->Su = Su_Max;
			    Write_Filter_Parm(0x00 + Band, Data_Values_Pointer->Fu, Data_Values_Pointer->Su, Sample_Rate,0 ,0); 
				Display_Value = Data_Values_Pointer->Su;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Del_Parm_Fn;
				Display_Value = Data_Values_Pointer->Delay_mm;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}

			if Exit_Key
				Display_Value = XO_Parm;
			else;

		}

		/************************************************************************/
		/* Delay for specific Crossover Band									*/
		/************************************************************************/
		else if(Sel_Fn == XO_Del_Parm_Fn)
		{
			/*		This section reads user input for delay for the band's		*/
			/*		XO parameters	 											*/
			/* Up 																*/
			if Up_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = Fast_Chane_Delays;
				else
					Step_Size = 3;
				Data_Values_Pointer->Delay_mm = Data_Values_Pointer->Delay_mm + Step_Size;
				if (Data_Values_Pointer->Delay_mm > Delay_mm_Max) Data_Values_Pointer->Delay_mm = Delay_mm_Max;
				Write_Delay(Band, Data_Values_Pointer->Delay_mm, Sample_Rate);
				Display_Value = Data_Values_Pointer->Delay_mm;
			}
			/* Down */
			else if Down_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = -Fast_Chane_Delays;
				else
					Step_Size = -3;
				Data_Values_Pointer->Delay_mm = Data_Values_Pointer->Delay_mm + Step_Size;
				if (Data_Values_Pointer->Delay_mm < Delay_mm_Min) Data_Values_Pointer->Delay_mm = Delay_mm_Min;
				Write_Delay(Band, Data_Values_Pointer->Delay_mm, Sample_Rate);
				Display_Value = Data_Values_Pointer->Delay_mm;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Att_Parm_Fn;
				Display_Value = Data_Values_Pointer->Atten;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}

			if Exit_Key
				Display_Value = XO_Parm;
			else;

		}

		/************************************************************************/
		/* Crossover Attenuation for band										*/
		/************************************************************************/
		else if(Sel_Fn == XO_Att_Parm_Fn)
		{
			/*		This section reads user input for the band's attenuation for*/
			/*		XO parameters	 											*/
			/* Up 																*/		
			if Up_Keys
			{
				Step_Size = 1;
				Data_Values_Pointer->Atten += Step_Size;
				if (Data_Values_Pointer->Atten > Atten_Max) Data_Values_Pointer->Atten = Atten_Max;
				Update_Atten_Set(DAC_Atten);
				Display_Value = Data_Values_Pointer->Atten;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = 1;
				if (Data_Values_Pointer->Atten < (unsigned char)Step_Size)
					Data_Values_Pointer->Atten = Atten_Min;				
				else
					Data_Values_Pointer->Atten -= Step_Size;				
				Update_Atten_Set(DAC_Atten);
				Display_Value = Data_Values_Pointer->Atten;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Inv_Parm_Fn;
				Display_Value = Data_Values_Pointer->Invert;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}
			
			if Exit_Key
				Display_Value = XO_Parm;
			else;
		}


		/************************************************************************/
		/* Crossover Invert Signal for band										*/
		/************************************************************************/
		else if(Sel_Fn == XO_Inv_Parm_Fn)
		{
			/*		This section reads user input for the band's inversion for  */
			/* Up 																*/		
			if Up_Keys
			{
				Step_Size = 1;
				if (Data_Values_Pointer->Invert == Invert_True)
					Data_Values_Pointer->Invert = Invert_False;				
				else
					Data_Values_Pointer->Invert = Invert_True;				
				Write_Invert(Band, Data_Values_Pointer->Invert);
				Display_Value = Data_Values_Pointer->Invert;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = 1;
				if (Data_Values_Pointer->Invert == Invert_True)
					Data_Values_Pointer->Invert = Invert_False;				
				else
					Data_Values_Pointer->Invert = Invert_True;				
				Write_Invert(Band, Data_Values_Pointer->Invert);
				Display_Value = Data_Values_Pointer->Invert;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_LF_Parm_Fn;
				if (Band < Max_Bands)
					Band += 1;
				else
					Band = 0;   /* always start from band zero */
				/* Use pointer to Data_Values array to save code space 			*/
				/* Just don't go changing Band without updating this pointer!!! */
				Data_Values_Pointer = &(Data_Values[Band]);
				Display_Value = Data_Values_Pointer->Fl;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = XO_Parm_Sel;
			}
			
			if Exit_Key
				Display_Value = XO_Parm;
			else;
		}




		/************************************************************************/
		/* Currently in EQ mode - look after key presses and function changes 	*/
		/* This section chooses which Parametric EQ the user is to change       */
		/************************************************************************/
		else if(Sel_Fn == EQ_Sel_Fn)
		{
			/*	This section works out which band the user wants to set 
				EQ parameters for 											*/
			/* Up */
			if Up_Keys
			{
				if (EQ_Parm < Gen3_Para_data)
					EQ_Parm += 1;
				else
					EQ_Parm = Band_1_Para_data;   /* always start from band zero */
				Display_Value = EQ_Parm;
			}
			/* Down */
			else if Down_Keys
			{
				if (EQ_Parm > 0)
					EQ_Parm -= 1;
				else
					EQ_Parm = Gen3_Para_data;
				Display_Value = EQ_Parm;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				Sel_Fn = EQ_Type_Sel_Fn;
				Band = EQ_Parm;
				/* Use pointer to Data_Values array to save code space 			*/
				/* Just don't go changing Band without updating this pointer!!! */
				EQ_Values_Pointer = &(EQ_Values[Band]);				
				Display_Value = EQ_Values_Pointer->Type;
			}
			/* Exit */
			else if Exit_Key
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
				Band = 0;
				/* Use pointer to Data_Values array to save code space 			*/
				/* Just don't go changing Band without updating this pointer!!! */
				Data_Values_Pointer = &(Data_Values[Band]);
			}
			else;
		}

			
		/************************************************************************/
		/* Parametric EQ Centre Frequency Select Fcuntion						*/
		/************************************************************************/
		else if(Sel_Fn == EQ_Freq_Sel_Fn)
		{
			/*	This section reads user input for centre frequency 				*/
			/*	XO parameters for Parametric EQ									*/
			/* Up 																*/
			if Up_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = Fast_Change_Frequencies;
				else
					Step_Size = 10;
				EQ_Values_Pointer->CF = EQ_Values_Pointer->CF + Step_Size;
				if (EQ_Values_Pointer->CF > EQ_Max) EQ_Values_Pointer->CF = EQ_Max;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->CF;
			}
			/* Down */
			else if Down_Keys
			{
				if (Fast_Change > (unsigned int)1)
					Step_Size = -Fast_Change_Frequencies;
				else
					Step_Size = -10;
				EQ_Values_Pointer->CF = EQ_Values_Pointer->CF + Step_Size;
				if (EQ_Values_Pointer->CF < EQ_Min) EQ_Values_Pointer->CF = EQ_Min;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->CF;
		}
			/* Select Key hit */
			else if Sel_Key
			{
				if (EQ_Values_Pointer->Type == HiShelf_EQ)
				{
					Step_Size = 0;  
					Sel_Fn = EQ_Type_Sel_Fn;
					EQ_Parm += 1;
					if (EQ_Parm > Gen2_Para_data)
						EQ_Parm = Band_1_Para_data;
					Band = EQ_Parm;
					/* Use pointer to Data_Values array to save code space 			*/
					/* Just don't go changing Band without updating this pointer!!! */
					EQ_Values_Pointer = &(EQ_Values[Band]);
					Display_Value = EQ_Values_Pointer->Type;
				}
				else
				{
					Step_Size = 0;
					Sel_Fn = EQ_Q_Sel_Fn;
					Display_Value = EQ_Values_Pointer->Q;
				}
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = EQ_Sel_Fn;
			}
			else;
			

			if Exit_Key
				Display_Value = EQ_Parm;
			else;
		}


		/************************************************************************/
		/* Parametric EQ Q Value Select Funtion									*/
		/* Note that Q is a signed char, scaled by 10,							*/
		/* Value of variable is -100 to 100, representing -10.0 to +10.0 		*/	
		/************************************************************************/
		else if(Sel_Fn == EQ_Q_Sel_Fn)
		{
			/*	This section reads user input for the Q value	 				*/
			/*	for Parametric EQ												*/
			/* Up 																*/
			if Up_Keys
			{
				Step_Size = Fast_Change;
				EQ_Values_Pointer->Q = EQ_Values_Pointer->Q + Step_Size;
				if (EQ_Values_Pointer->Q > EQ_Q_Max) EQ_Values_Pointer->Q = EQ_Q_Max;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->Q;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = -1 * (int)Fast_Change;				
				EQ_Values_Pointer->Q = EQ_Values_Pointer->Q + Step_Size;
				if (EQ_Values_Pointer->Q < EQ_Q_Min) EQ_Values_Pointer->Q = EQ_Q_Min;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->Q;
		}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;
				Sel_Fn = EQ_Gain_Sel_Fn;
				Display_Value = EQ_Values_Pointer->Gain;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = EQ_Sel_Fn;
			}
			else;
			

			if Exit_Key
				Display_Value = EQ_Parm;
			else;
		}


		/************************************************************************/
		/* Parametric EQ Gain Value Select Funtion								*/
		/* Value in dB															*/
		/************************************************************************/
		else if(Sel_Fn == EQ_Gain_Sel_Fn)
		{
			/*	This section reads user input for the Gain value				*/
			/*	for Parametric EQ												*/
			/* Up 																*/
			if Up_Keys
			{
				Step_Size = 1;
				EQ_Values_Pointer->Gain = EQ_Values_Pointer->Gain + Step_Size;
				if (EQ_Values_Pointer->Gain > EQ_Gain_Max) EQ_Values_Pointer->Gain = EQ_Gain_Max;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->Gain;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = -1;
				EQ_Values_Pointer->Gain = EQ_Values_Pointer->Gain + Step_Size;
				if (EQ_Values_Pointer->Gain < EQ_Gain_Min) EQ_Values_Pointer->Gain = EQ_Gain_Min;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->Gain;
		}
			/* Select Key hit */
			else if Sel_Key
			{
				Step_Size = 0;  
				Sel_Fn = EQ_Type_Sel_Fn;
				EQ_Parm += 1;
				if (EQ_Parm > Gen3_Para_data)
					EQ_Parm = Band_1_Para_data;
				Band = EQ_Parm;
				/* Use pointer to Data_Values array to save code space 			*/
				/* Just don't go changing Band without updating this pointer!!! */
				EQ_Values_Pointer = &(EQ_Values[Band]);
				Display_Value = EQ_Values_Pointer->Type;			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = EQ_Sel_Fn;
			}
			else;
			

			if Exit_Key
				Display_Value = EQ_Parm;
			else;
		}

		/************************************************************************/
		/* Parametric EQ Type Select Funtion									*/
		/************************************************************************/
		else if(Sel_Fn == EQ_Type_Sel_Fn)
		{
			/*	This section reads user input for the EQ Type Desired			*/
			/*	for Parametric EQ												*/
			/* Up 																*/
			if Up_Keys
			{
				Step_Size = 1;
				EQ_Values_Pointer->Type = EQ_Values_Pointer->Type + Step_Size;
				if (EQ_Values_Pointer->Type > HiShelf_EQ) EQ_Values_Pointer->Type = No_EQ;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->Type;
			}
			/* Down */
			else if Down_Keys
			{
				Step_Size = -1;
				EQ_Values_Pointer->Type = EQ_Values_Pointer->Type + Step_Size;
				if (EQ_Values_Pointer->Type < No_EQ) EQ_Values_Pointer->Type = HiShelf_EQ;
    			Write_Filter_Parm(Band, EQ_Values_Pointer->CF, EQ_Values_Pointer->Type + 0x10, Sample_Rate,EQ_Values_Pointer->Q ,EQ_Values_Pointer->Gain); 
				Display_Value = EQ_Values_Pointer->Type;
		}
			/* Select Key hit */
			else if Sel_Key
			{
				if (EQ_Values_Pointer->Type == No_EQ)
				{
					Step_Size = 0;  
					Sel_Fn = EQ_Type_Sel_Fn;
					EQ_Parm += 1;
					if (EQ_Parm > Gen3_Para_data)
						EQ_Parm = Band_1_Para_data;
					Band = EQ_Parm;
					/* Use pointer to Data_Values array to save code space 			*/
					/* Just don't go changing Band without updating this pointer!!! */
					EQ_Values_Pointer = &(EQ_Values[Band]);
					Display_Value = EQ_Values_Pointer->Type;
				}
			else
				{
					Step_Size = 0;
					Sel_Fn = EQ_Freq_Sel_Fn;
					/* Use pointer to Data_Values array to save code space 			*/
					/* Just don't go changing Band without updating this pointer!!! */
					EQ_Values_Pointer = &(EQ_Values[Band]);
					Display_Value = EQ_Values_Pointer->CF;
				}
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = EQ_Sel_Fn;
			}
			else;

			if Exit_Key
				Display_Value = EQ_Parm;
			else;
		}




		/********************************************************************/
		/* Currently in Input Select Function - act on legal button presses */
		/********************************************************************/
		else if (Sel_Fn == In_Sel_Fn)
		{
			/* Up */
			if Down_Keys
			{
				if (Input_Selected == Input_SPDIF) 
				{
					Input_Selected = Input_ADC;
 				}	
				else
				{
					Input_Selected = Input_SPDIF;
				}
				Display_Value = Input_Selected;
				Write_Input(Input_Selected);
			}
			/* Down */
			else if Up_Keys
			{
				if (Input_Selected == Input_ADC) 
				{
					Input_Selected = Input_SPDIF;
 				}	
				else
				{
					Input_Selected = Input_ADC;
				}
				Display_Value = Input_Selected;
				Write_Input(Input_Selected);
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				EEPROM_WRITE(Input_Selected_Offset, Input_Selected);
				Display_Value = Mode;
			}
			else;
		}





		/************************************************************************/
		/* Currently in SAVE Function, act on legal button presses				*/
		/************************************************************************/
		else if(Sel_Fn == Save_Sel_Fn)
		{
			/*		This section miplements the save function 					*/
			/* Up 																*/
			if Up_Keys
			{
				Memory_Bank += 1;
				if (Memory_Bank > Max_Mem_Banks)
					Memory_Bank = 0;
				Display_Value = Memory_Bank;
			}
			/* Down */
			else if Down_Keys
			{
				if (Memory_Bank == 0)
					Memory_Bank = Max_Mem_Banks;
				else
					Memory_Bank -= 1;
				Display_Value = Memory_Bank;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				/********************************************************************/
				/*		Save the data 
																					*/
				/********************************************************************/		
				for (loop=0; loop < 4; loop++)
					{
						BandTemp = (loop*Band_Data_Array_Size) + (ParmSet_Array_Size * Memory_Bank);
						Data_Values_Pointer = &(Data_Values[loop]);
					/* lower Freq */
					EEPROM_WRITE((BandTemp + Fl_Offset), (char) (Data_Values_Pointer->Fl));
					EEPROM_WRITE((BandTemp + Fl_Offset + 1), (char) (Data_Values_Pointer->Fl / 256));
					/* Upper Freq*/
					EEPROM_WRITE((BandTemp + Fu_Offset), (char)(Data_Values_Pointer->Fu));
					EEPROM_WRITE((BandTemp + Fu_Offset + 1), (char)(Data_Values_Pointer->Fu / 256 ));
					/* Lower Slope */
					EEPROM_WRITE((BandTemp + Sl_Offset), Data_Values_Pointer->Sl);
					/* Upper Slope */
					EEPROM_WRITE((BandTemp + Su_Offset), Data_Values_Pointer->Su);
					/* Attenuation for band */
					EEPROM_WRITE((BandTemp + Atten_Offset), Data_Values_Pointer->Atten);
					/* Delay for band */
					EEPROM_WRITE((BandTemp + Delay_mm_Offset), (char) (Data_Values_Pointer->Delay_mm ));
					EEPROM_WRITE((BandTemp + Delay_mm_Offset + 1), (char) (Data_Values_Pointer->Delay_mm / 256));
					EEPROM_WRITE((BandTemp + Invert_Offset), Data_Values_Pointer->Invert);
									
					EQ_Values_Pointer = &(EQ_Values[3*loop]);	
					/* Parametric EQ Centre Freq */
					EEPROM_WRITE((BandTemp + Para_CF_Offset), (char) (EQ_Values_Pointer->CF));
	 				EEPROM_WRITE((BandTemp + Para_CF_Offset + 1), (char) (EQ_Values_Pointer->CF/256));
					/* Parametric EQ Q Value */
					EEPROM_WRITE((BandTemp + Para_Q_Offset), (char) (EQ_Values_Pointer->Q));
					/* Parametric EQ Gain */
					EEPROM_WRITE((BandTemp + Para_Gain_Offset), (char)(EQ_Values_Pointer->Gain));
					/* Parametric EQ Type */
					EEPROM_WRITE((BandTemp + Para_Type_Offset), (char)(EQ_Values_Pointer->Type));

					EQ_Values_Pointer = &(EQ_Values[(3*loop) + 1]);	
					/* Parametric EQ Centre Freq */
					EEPROM_WRITE((BandTemp + Para_CF_1_Offset), (char) (EQ_Values_Pointer->CF));
	 				EEPROM_WRITE((BandTemp + Para_CF_1_Offset + 1), (char) (EQ_Values_Pointer->CF/256));
					/* Parametric EQ Q Value */
					EEPROM_WRITE((BandTemp + Para_Q_1_Offset), (char) (EQ_Values_Pointer->Q));
					/* Parametric EQ Gain */
					EEPROM_WRITE((BandTemp + Para_Gain_1_Offset), (char)(EQ_Values_Pointer->Gain));
					/* Parametric EQ Type */
					EEPROM_WRITE((BandTemp + Para_Type_1_Offset), (char)(EQ_Values_Pointer->Type));

					EQ_Values_Pointer = &(EQ_Values[(3*loop) + 2]);	
					/* Parametric EQ Centre Freq */
					EEPROM_WRITE((BandTemp + Para_CF_2_Offset), (char) (EQ_Values_Pointer->CF));
	 				EEPROM_WRITE((BandTemp + Para_CF_2_Offset + 1), (char) (EQ_Values_Pointer->CF/256));
					/* Parametric EQ Q Value */
					EEPROM_WRITE((BandTemp + Para_Q_2_Offset), (char) (EQ_Values_Pointer->Q));
					/* Parametric EQ Gain */
					EEPROM_WRITE((BandTemp + Para_Gain_2_Offset), (char)(EQ_Values_Pointer->Gain));
					/* Parametric EQ Type */
					EEPROM_WRITE((BandTemp + Para_Type_2_Offset), (char)(EQ_Values_Pointer->Type));

					};

				/* Input */			
				EEPROM_WRITE(Input_Selected_Offset, Input_Selected);
				/* Volume */			
				EEPROM_WRITE(Volume_Offset, DAC_Atten);
				
				/* Read Data for each Para Set from EEPROM */
				BandTemp = (Para_Gen_Base + Para_Parmset_Size * Memory_Bank);	

				EQ_Values_Pointer = &(EQ_Values[Gen1_Para_data]);
	
					/* Gen1 Parametric EQ Centre Freq */
				EEPROM_WRITE((BandTemp + Para_Gen1_CF_Offset), (char) (EQ_Values_Pointer->CF));
				EEPROM_WRITE((BandTemp + Para_Gen1_CF_Offset + 1), (char)(EQ_Values_Pointer->CF/256));
				/* Gen1 Parametric EQ Q Value */
				EEPROM_WRITE((BandTemp + Para_Gen1_Q_Offset), (char)(EQ_Values_Pointer->Q));
				/* Gen1 Parametric EQ Gain */
				EEPROM_WRITE((BandTemp + Para_Gen1_Gain_Offset), (char)(EQ_Values_Pointer->Gain));
				/* Gen1 Parametric EQ Type */
				EEPROM_WRITE((BandTemp + Para_Gen1_Type_Offset), (char)(EQ_Values_Pointer->Type));

				EQ_Values_Pointer = &(EQ_Values[Gen2_Para_data]);
				/* Gen2 Parametric EQ Centre Freq */
				EEPROM_WRITE((BandTemp + Para_Gen2_CF_Offset), (char)(EQ_Values_Pointer->CF));
				EEPROM_WRITE((BandTemp + Para_Gen2_CF_Offset + 1), (char)(EQ_Values_Pointer->CF/256));
				/* Gen1 Parametric EQ Q Value */
				EEPROM_WRITE((BandTemp + Para_Gen2_Q_Offset), (char)(EQ_Values_Pointer->Q));
				/* Gen1 Parametric EQ Gain */
				EEPROM_WRITE((BandTemp + Para_Gen2_Gain_Offset), (char)(EQ_Values_Pointer->Gain));
				/* Gen1 Parametric EQ Type */
				EEPROM_WRITE((BandTemp + Para_Gen2_Type_Offset), (char)(EQ_Values_Pointer->Type));

				EQ_Values_Pointer = &(EQ_Values[Gen3_Para_data]);
				/* Gen2 Parametric EQ Centre Freq */
				EEPROM_WRITE((BandTemp + Para_Gen3_CF_Offset), (char)(EQ_Values_Pointer->CF));
				EEPROM_WRITE((BandTemp + Para_Gen3_CF_Offset + 1), (char)(EQ_Values_Pointer->CF/256));
				/* Gen1 Parametric EQ Q Value */
				EEPROM_WRITE((BandTemp + Para_Gen3_Q_Offset), (char)(EQ_Values_Pointer->Q));
				/* Gen1 Parametric EQ Gain */
				EEPROM_WRITE((BandTemp + Para_Gen3_Gain_Offset), (char)(EQ_Values_Pointer->Gain));
				/* Gen1 Parametric EQ Type */
				EEPROM_WRITE((BandTemp + Para_Gen3_Type_Offset), (char)(EQ_Values_Pointer->Type));

				Sel_Fn = Choosing_Fn;
				Mode = Volume_Mode;
				Display_Value = Mode;
			}
			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Mode = Volume_Mode;
				Display_Value = Mode;
			}
			else;
		}

		/************************************************************************/
		/* Currently in LOAD Function, act on legal button presses 				*/
		/************************************************************************/
		else if(Sel_Fn == Load_Sel_Fn)
		{
			/********************************************************************/
			/*		This section miplements the load data function 				*/
			/********************************************************************/
			/* Up */
			if Up_Keys
			{
				Memory_Bank += 1;
				if (Memory_Bank > Max_Mem_Banks)
					Memory_Bank = 0;
				Display_Value = Memory_Bank;
			}
			/* Down */
			else if Down_Keys
			{
				if (Memory_Bank == 0)
					Memory_Bank = Max_Mem_Banks;
				else
					Memory_Bank -= 1;
				Display_Value = Memory_Bank;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				/********************************************************************/
				/*		Load the data 												*/
				/********************************************************************/		
				Load_From_Memory(Memory_Bank); /* Read Data from ROM */
				/* now program the values the user loaded...*/
				Update_Atten_Set(Mute);
				if (DAC_Atten < Atten_On_Loading_Min)
						DAC_Atten = Atten_On_Loading_Min;			/* limit things just in case there is a massive error by the user... */
				Update_Filter_Set(Sample_Rate);						/* Load Filter Settings */
				Update_Delay_Set(Sample_Rate);						/* Load Delay Settings */
				Update_Atten_Set(DAC_Atten);									/* Load Attenuator Settings including volume */

				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}

		/************************************************************************/
		/* Exit 																*/
		/************************************************************************/
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			
			else
				Update_Display = 0;
		}
	
		if (Update_Display)	
			Display_Data(Band, Sel_Fn, Display_Value);
		else;


	}  /* Done changing values for the user */


	/* Clears the flag saying that the push button press has been addressed */
	Step_Size = 0;

	Auto_Revert = Auto_Revert - 1;   				/* If no key pressed, then decrement this counter */
	if 	(Auto_Revert == 0)   						/* Then force the system to volume mode */
		{
			Auto_Revert = Auto_Revert_Init_Val;		/* If key pressed, reset the counter */
			Band = 0;
			Sel_Fn = Vol_Sel_Fn;
			Display_Value = DAC_Atten;
			Display_Data(Band, Sel_Fn, Display_Value);
			Display_Data(Band, Sel_Fn, Display_Value);
		}
		

	}
while (1);
}
