/* 28/6/07													*/
/* End of build 1 of major rewrite to implement rotary ctrl */
/* basic functionality for:
	- Load and save three parm sets
	- Rotary control basics
	- Memory map for load save
	- menu structure for rotary control
*/
/* Build currently excludes
	- Parametric EQ
*/

/*
 * Bit masks for microconroller I/O
 * 
*/

/* 17/1/09													*/
/* Fix EEPROM Read bug for new micros						 */


/* Input port bits */
/* Input Port E */
#define Band_Select 1
#define Function_Select 2
#define Max_Bands  3

/* Output Port D */
/* 
 * Drive LCD Data to dispaly select using port D Output
 */
#define Clear_LCD 1
#define Home_LCD 2
#define No_Blink_LCD 8
#define Blink_LCD 14

/* Function Select 							*/
/* 	RS	RW 	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	1	1	1	0	0	0	*/
#define LCD_Function 0x38

/* Display On */
/* 	RS	RW 	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	1	1	1	1	*/
#define LCD_Display_On 0x0F

/* Display Clear */
/* 	RS	RW 	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	0	0	0	1	*/
#define LCD_Display_Clear 0x01

/* Entry Mode */
/* 	RS	RW 	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	0	1	1	1	*/
#define LCD_Display_Entry_Mode 0x06

/* DDRAM Line 1 Address */
/* 	RS	RW 	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	1	0	0	0	0	0	0	0	*/
#define LCD_Line_1 0x80

/* DDRAM Line 2 Address */
/* 	RS	RW 	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	1	1	0	0	0	0	0	0	*/
#define LCD_Line_2 0xC0


/* Enable Write Data to LCD */
/* 	RS	RW 	E	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	1	0	0	d7	d6	d5	d4	d3	d2	d1	d0	*/
#define Write_Data_Enable_LCD 8


/* Write Data to LCD */
/* 	RS	RW 	E	DB7 DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	1	0	1	d7	d6	d5	d4	d3	d2	d1	d0	*/
#define Write_Data_LCD 12

/* Output Port B */
/* 
 * Drive LCD Control for dispaly select using port B Output
 */
#define Enable_LCD 4
#define R_W_LCD 16
#define PGA2320_Select 32
#define RS_LCD 8
#define Write_Control_LCD 0
#define Write_Control_Enable_LCD 4



/* Display Strings */
#define Blank_Char_Init 		0x20
#define Line1_Init      		"  TGM Was Here  "
#define Line2_Init     	 		" ADAU1442 V2-00 "
#define Line2_Select_Init		"Press Select:   "
#define Line1_EQ_Init   		"EQ Setup: "
#define Line1_XO_Init   		"XO Setup Band:  "
#define Line1_Save_Init 		"Save to bank:   "
#define Line1_Load_Init 		"Load from bank: "
#define Line1_Vol_Init  		"Volume:       dB"
#define Line2_Vol_Init  		"Ref Max Output  "
#define Line1_Input_Init  		"Input Selected: "
#define Slopes_Def_Init {{78, 111, 110,101},{66, 119, 54, 32},{66, 119, 49, 50},{76, 82, 50, 52},{76, 82, 52, 56}}
#define Boot_Msg 666
#define Line1_Choose_Str_Init	"Select Function:"


/*
Stuff to allow modes to be tracked
Use these to control the state machine
*/
#define Choosing_Fn		0
#define Volume_Mode 	1
#define XO_Mode 		2
#define Save_Mode		3
#define Load_Mode		4
#define EQ_Mode			5
#define In_Sel_Mode		6

/*
Stuff to allow functions to be tracked in Volume mode
*/
#define Vol_Sel_Fn		10

/*
Stuff to allow functions to be tracked in XO mode
*/

#define XO_Band_Sel_Fn 	20
#define XO_Parm_Sel		21
#define XO_LF_Parm_Fn	22
#define XO_HF_Parm_Fn	23
#define XO_SL_Parm_Fn	24
#define XO_SU_Parm_Fn	25
#define XO_Del_Parm_Fn	26
#define XO_Att_Parm_Fn	27
#define XO_Inv_Parm_Fn	28

/*
Stuff to allow functions to be tracked in Save mode
*/
#define Save_Sel_Fn 	30

/*
Stuff to allow functions to be tracked in Load mode
*/
#define Load_Sel_Fn 	40

/*
Stuff to allow functions to be tracked in EQ mode
*/
#define EQ_Sel_Fn 			50
#define EQ_Freq_Sel_Fn 		51
#define EQ_Q_Sel_Fn  		52
#define EQ_Gain_Sel_Fn 		53
#define EQ_Type_Sel_Fn 		54

/*
Stuff to allow functions to be tracked in Input_Select mode
*/
#define In_Sel_Fn			60
#define SPDIF_Sel_Fn		61
#define ADC_Sel_Fn 			62


#define No_EQ					0
#define Para_EQ					1
#define HiShelf_EQ				2


/*
Stuff to allow memory bank in use to be tracked
*/
#define Max_Mem_Banks 	2
#define Default_Mem_Bank 	0

/*
more stuff 
Use this bitfield to watch what buttons are happening...
*/
#define  Vol_Dn (unsigned char) 1
#define  Vol_Up (unsigned char) 2
#define  Exit (unsigned char) 2
#define  Vals_0 (unsigned char) 8
#define  Vals_1 (unsigned char) 16
#define  Vals_2 (unsigned char) 32
#define  Mode_Sel (unsigned char) 64
#define  Sel (unsigned char) 4

#define Slow_Counter_Init 80
#define Fast_Counter_Init 15

#define Phase_0 (unsigned char) 0
#define Phase_1 Vals_0
#define Phase_2 Vals_0 + Vals_1
#define Phase_3 Vals_1

#define Rot_Up Vals_1
#define Rot_Down Vals_0 + Vals_1

#define Up_Keys (Keypress_Read == Vals_1)
#define Down_Keys (Keypress_Read == (Vals_0 + Vals_1))
#define Sel_Key (Keypress_Read == Sel)
#define Exit_Key (Keypress_Read == Exit)

/*Define Value for Mute Volume */
#define  Mute 255


/* Auto Repeat Counts - how long to wait before starting to Auto Repeat*/
#define Auto_Repeat_Count 6

#define Port_E_Button_Pressed 3
#define Port_A_Button_Pressed (1+2+4+8+16+32)


typedef struct{
		int Fl;
		int Fu;
		signed char Sl;
		signed char Su;
		unsigned char Atten;
		int Delay_mm;
		char Invert;
}  data_val_struct ;


typedef struct{
		int CF;
		signed char Q;
		signed char Gain;
		signed char Type;
}  Para_EQ_struct ;


#define Band_1_Para_data 0
#define Gen1_Para_data 12
#define Gen2_Para_data 13
#define Gen3_Para_data 14

/* crossover and para data locations */
#define Fl_Offset 0
#define Fu_Offset 2
#define Sl_Offset 4
#define Su_Offset 5
#define Atten_Offset 6
#define Delay_mm_Offset 7
#define Para_CF_Offset 9
#define Para_Q_Offset 11
#define Para_Gain_Offset 12
#define Para_Type_Offset 13
#define Para_CF_1_Offset 14
#define Para_Q_1_Offset 16
#define Para_Gain_1_Offset 17
#define Para_Type_1_Offset 18
#define Para_CF_2_Offset 19
#define Para_Q_2_Offset 21
#define Para_Gain_2_Offset 22
#define Para_Type_2_Offset 23
#define Invert_Offset 24
#define Band_Data_Array_Size 25
#define ParmSet_Array_Size 100
#define Input_Selected_Offset 254
#define Volume_Offset 255


#define Para_Gen_Base (200)
#define Para_Parmset_Size 15

#define Para_Gen1_CF_Offset 0
#define Para_Gen1_Q_Offset 2
#define Para_Gen1_Gain_Offset 3
#define Para_Gen1_Type_Offset 4

#define Para_Gen2_CF_Offset 5
#define Para_Gen2_Q_Offset 7
#define Para_Gen2_Gain_Offset 8
#define Para_Gen2_Type_Offset 9

#define Para_Gen3_CF_Offset 10
#define Para_Gen3_Q_Offset 12
#define Para_Gen3_Gain_Offset 13
#define Para_Gen3_Type_Offset 14

/*  Limit Values for Data Structures  */
#define	EQ_Min 10
#define	EQ_Gain_Max 10
#define	EQ_Gain_Min -10
#define	EQ_Q_Max 100
#define	EQ_Q_Min 1
#define	EQ_Max 15000
#define	Fl_Min 10
#define	Fl_Max 15000
#define	Fu_Min 10
#define	Fu_Max 15000
#define	Sl_Min 0    /* Sl = 0, None */
				    /* Sl = 1, 6dB Slope */
				    /* Sl = 2, 12dB Slope */
				    /* Sl = 3, LR24 Slope */
				    /* Sl = 4, LR48 Slope */
#define	Sl_Max 4    /* LR 48 Slope */
#define	Su_Min 0    /* Sl = 0, None */
				    /* Sl = 1, 6dB Slope */
				    /* Sl = 2, 12dB Slope */
				    /* Sl = 3, LR24 Slope */
				    /* Sl = 4, LR48 Slope */
#define	Su_Max 4    /* LR 48 Slope */
#define	Atten_Min (unsigned char) 0
#define	Atten_Max (unsigned char) 40
#define	Delay_mm_Min 1
/* Note this maximum is determined by the RAM allocated in the DSP */
/* Current allocation is 900 samples, or 6397mm odd */
/* This parameter = (900 / Sample_Rate)*Speed_Of_Sound_in_air */
#define	Delay_mm_Max 6370
#define Invert_True 1
#define Invert_False 0
#define Volume_Atten_Max 255
#define Volume_Atten_Min 0
#define Atten_On_Loading_Min (unsigned char)80

#define Input_SPDIF (unsigned char) 0
#define Input_ADC (unsigned char)1

#define Auto_Revert_Init_Val 4000000

/* 
PORT C definitions
*/
#define Reset_DSP       128   /* Bit 7 of port C */ 
#define Reset_DAC 		64   /* bit 6 of port C */
#define SDI_Out         32   /* Bit 5 of port C */ 
#define SDI_In PORTCbits.RC4
#define SCLK            8   /* Bit 3 of port C */ 
#define CSel            4   /* Bit 2 of port C */ 
#define EEPROM_WR_Delay 10  /* Approx EEPROM write delay in mS */ 



/* Addresses of things in the system... */
#define Write_SPI 0X00

/* Register Addresses in ADAU1442 */
#define ADAU1442_Safeload_Base_Parameter 0x0001
#define ADAU1442_Safeload_Target_Address 0x0006
#define ADAU1442_Safeload_Number_Of_Words 0x0007



/* Can't believe I have to define this!!! */
#define Pi (double) 3.141592653590
#define sqrt2 (double) 1.414213562373

/* Savy typing and make things clearer in the code */
#define ClearDSPReset PORTC = PORTC | Reset_DSP
#define SetDSPReset PORTC = PORTC & (0xFF - Reset_DSP)
#define ClearDACReset PORTC = PORTC | Reset_DAC
#define SetDACReset PORTC = PORTC & (0xFF - Reset_DAC)
#define DAC_Reset_Time 20
#define More_Than_1_Audio_Frame_uS 32
#define CselLow DelayUs(1); PORTC = PORTC & (0xFF - CSel); DelayUs(1)
#define CselHigh DelayUs(1); PORTC = PORTC | (CSel); DelayUs(1)
#define CselLowPGA2320 DelayUs(1); PORTB = PORTB & (0xFF - PGA2320_Select); DelayUs(1)
#define CselHighPGA2320 DelayUs(1); PORTB = PORTB | (PGA2320_Select); DelayUs(1)
#define Write_Control_LCD_CSEL DelayUs(1); PORTB = (Write_Control_LCD + PGA2320_Select); DelayUs(1)
#define Write_Control_Enable_LCD_CSEL DelayUs(1); PORTB = (Write_Control_Enable_LCD + PGA2320_Select); DelayUs(1)
#define Write_Data_LCD_CSEL DelayUs(1); PORTB = (Write_Data_LCD + PGA2320_Select); DelayUs(1)
#define Write_Data_Enable_LCD_CSEL DelayUs(1); PORTB = (Write_Data_Enable_LCD + PGA2320_Select); DelayUs(1)


#define SClkLow PORTC = PORTC & (0xFF - SCLK)
#define SClkHigh PORTC = PORTC | (SCLK)
#define Write_8_Bits(Data_Word) WriteSPI(Data_Word,8)
#define Write_16_Bits(Data_Word) WriteSPI(Data_Word,16)
#define Write_32_Bits(Data_Word) WriteSPI(Data_Word,32)
#define Write_SPI_Addr(Device_Address, Write_Bit) WriteSPI(((Device_Address << 1)+Write_Bit),8)
#define Write_To_SPI 0
#define Read_From_SPI 1
#define Fast_Change_Frequencies 200
#define Fast_Chane_Delays 100


#define SpeedOfSound 341210   /* millimetres per second */
#define A1 IIR_B0_B1_B2_A1_A2[3]
#define A2 IIR_B0_B1_B2_A1_A2[4]
#define B0 IIR_B0_B1_B2_A1_A2[0]
#define B1 IIR_B0_B1_B2_A1_A2[1]
#define B2 IIR_B0_B1_B2_A1_A2[2]

#define A1_2 IIR2_B0_B1_B2_A1_A2[3]
#define A2_2 IIR2_B0_B1_B2_A1_A2[4]
#define B0_2 IIR2_B0_B1_B2_A1_A2[0]
#define B1_2 IIR2_B0_B1_B2_A1_A2[1]
#define B2_2 IIR2_B0_B1_B2_A1_A2[2]

/* number of bytes in single parameter transaction */
#define NumBytes_Word_Parms 4
#define Non_Inverted_Gain_Val 0x00800000
#define Inverted_Gain_Val 0xFF800000

void	Display_Char(char, char);


#define EEPROM_READ(addr) ((EEADR=(addr)),(EECON1bits.EEPGD=0),(EECON1bits.CFGS=0),(EECON1bits.RD=1),EEDATA)
void 	Print_Data(int, unsigned char*, unsigned int);
